#!

# This is statement is required by the build system to query build info
if __name__ == '__build__':
	raise Exception

import string
__version__ = string.split('$Revision: 1.1.1.1 $')[1]
__date__ = string.join(string.split('$Date: 2007/02/15 19:25:20 $')[1:3], ' ')
__author__ = 'Tarn Weisner Burton <twburton@users.sourceforge.net>'

#
# Ported to PyOpenGL 2.0 by Tarn Weisner Burton 10May2001
#
# This code was created by Richard Campbell '99 (ported to Python/PyOpenGL by John Ferguson 2000)
#
# The port was based on the PyOpenGL tutorial module: dots.py
#
# If you've found this code useful, please let me know (email John Ferguson at hakuin@voicenet.com).
#
# See original source and C based tutorial at http://nehe.gamedev.net
#
# Note:
# -----
# This code is not a good example of Python and using OO techniques.  It is a simple and direct
# exposition of how to use the Open GL API in Python via the PyOpenGL package.  It also uses GLUT,
# which in my opinion is a high quality library in that it makes my work simpler.  Due to using
# these APIs, this code is more like a C program using function based programming (which Python
# is in fact based upon, note the use of closures and lambda) than a "good" OO program.
#
# To run this code get and install OpenGL, GLUT, PyOpenGL (see http://www.python.org), and PyNumeric.
# Installing PyNumeric means having a C compiler that is configured properly, or so I found.  For
# Win32 this assumes VC++, I poked through the setup.py for Numeric, and chased through disutils code
# and noticed what seemed to be hard coded preferences for VC++ in the case of a Win32 OS.  However,
# I am new to Python and know little about disutils, so I may just be not using it right.
#
# BTW, since this is Python make sure you use tabs or spaces to indent, I had numerous problems since I
# was using editors that were not sensitive to Python.
#
from OpenGL.GL import *
from OpenGL.GLUT import *
from OpenGL.GLU import *
import sys

# Some api in the chain is translating the keystrokes to this octal string
# so instead of saying: ESCAPE = 27, we use the following.
ESCAPE = '\033'

# Number of the glut window.
window = 0

# A general OpenGL initialization function.  Sets all of the initial parameters.
def InitGL(Width, Height):				# We call this right after our OpenGL window is created.
    glClearColor(0.0, 0.0, 0.0, 0.0)	# This Will Clear The Background Color To Black
    glClearDepth(1.0)					# Enables Clearing Of The Depth Buffer
    glDepthFunc(GL_LESS)				# The Type Of Depth Test To Do
    glEnable(GL_DEPTH_TEST)				# Enables Depth Testing
    glShadeModel(GL_SMOOTH)				# Enables Smooth Color Shading

    glMatrixMode(GL_PROJECTION)
    glLoadIdentity()					# Reset The Projection Matrix
										# Calculate The Aspect Ratio Of The Window
    gluPerspective(45.0, float(Width)/float(Height), 0.1, 100.0)

    glMatrixMode(GL_MODELVIEW)

# The function called when our window is resized (which shouldn't happen if you enable fullscreen, below)
def ReSizeGLScene(Width, Height):
    if Height == 0:						# Prevent A Divide By Zero If The Window Is Too Small
	    Height = 1

    glViewport(0, 0, Width, Height)		# Reset The Current Viewport And Perspective Transformation
    glMatrixMode(GL_PROJECTION)
    glLoadIdentity()
    gluPerspective(45.0, float(Width)/float(Height), 0.1, 100.0)
    glMatrixMode(GL_MODELVIEW)

# The main drawing function.
def DrawGLScene():
	# Clear The Screen And The Depth Buffer
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
	glLoadIdentity()					# Reset The View

	# Move Left 1.5 units and into the screen 6.0 units.
	glTranslatef(-1.5, 0.0, -6.0)

	# Draw a triangle
	glBegin(GL_POLYGON)                 # Start drawing a polygon
	glVertex3f(0.0, 1.0, 0.0)           # Top
	glVertex3f(1.0, -1.0, 0.0)          # Bottom Right
	glVertex3f(-1.0, -1.0, 0.0)         # Bottom Left
	glEnd()                             # We are done with the polygon


	# Move Right 3.0 units.
	glTranslatef(3.0, 0.0, 0.0)

	# Draw a square (quadrilateral)
	glBegin(GL_QUADS)                   # Start drawing a 4 sided polygon
	glVertex3f(-1.0, 1.0, 0.0)          # Top Left
	glVertex3f(1.0, 1.0, 0.0)           # Top Right
	glVertex3f(1.0, -1.0, 0.0)          # Bottom Right
	glVertex3f(-1.0, -1.0, 0.0)         # Bottom Left
	glEnd()                             # We are done with the polygon

	#  since this is double buffered, swap the buffers to display what just got drawn.
	glutSwapBuffers()

# The function called whenever a key is pressed. Note the use of Python tuples to pass in: (key, x, y)
def keyPressed(*args):
	# If escape is pressed, kill everything.
    if args[0] == ESCAPE:
	    sys.exit()

def main():
	global window
	# For now we just pass glutInit one empty argument. I wasn't sure what should or could be passed in (tuple, list, ...)
	# Once I find out the right stuff based on reading the PyOpenGL source, I'll address this.
	glutInit(sys.argv)

	# Select type of Display mode:
	#  Double buffer
	#  RGBA color
	# Alpha components supported
	# Depth buffer
	glutInitDisplayMode(GLUT_RGBA | GLUT_DOUBLE | GLUT_DEPTH)

	# get a 640 x 480 window
	glutInitWindowSize(640, 480)

	# the window starts at the upper left corner of the screen
	glutInitWindowPosition(0, 0)

	# Okay, like the C version we retain the window id to use when closing, but for those of you new
	# to Python (like myself), remember this assignment would make the variable local and not global
	# if it weren't for the global declaration at the start of main.
	window = glutCreateWindow("Jeff Molofee's GL Code Tutorial ... NeHe '99")

   	# Register the drawing function with glut, BUT in Python land, at least using PyOpenGL, we need to
	# set the function pointer and invoke a function to actually register the callback, otherwise it
	# would be very much like the C version of the code.
	glutDisplayFunc(DrawGLScene)

	# Uncomment this line to get full screen.
	#glutFullScreen()

	# When we are doing nothing, redraw the scene.
	glutIdleFunc(DrawGLScene)

	# Register the function called when our window is resized.
	glutReshapeFunc(ReSizeGLScene)

	# Register the function called when the keyboard is pressed.
	glutKeyboardFunc(keyPressed)

	# Initialize our window.
	InitGL(640, 480)

	# Start Event Processing Engine
	glutMainLoop()

# Print message to console, and kick off the main to get it rolling.
print ("Hit ESC key to quit.")
main()



desc = {
    "adxl345": {
        "device_address": 83,
        "commands": {
            "init": [
                {
                    u"cmd": u"init",
                    u"device_address": 83
                },
                {
                    u"cmd": u"write_byte",
                    u"device_address": 83,
                    u"register": 45,
                    u"value": 0
                }, {
                    u"cmd": u"write_byte",
                    u"device_address": 83,
                    u"register": 45,
                    u"value": 8
                }, {
                    u"cmd": u"write_byte",
                    u"device_address": 83,
                    u"register": 49,
                    u"value": 8
                }, {
                    u"cmd": u"write_byte",
                    u"device_address": 83,
                    u"register": 49,
                    u"value": 3
                }],
            "read": [{
                "cmd": "read_block",
                "device_address": 83,
                "register": 50,
                "num_bytes": 6
            }]
        }
    },
    "tmp102": {
        "device_address": 72,
        "commands": {
            "read": [{
                "cmd": "read_block",
                "device_address": 72,
                "register": 50,
                "num_bytes": 2
            }]
        }
    }
}

instructions = desc["adxl345"]["commands"]["init"]
for x in instructions:
    print(x)


import time

import pigpio

pi = pigpio.pi()

pi.set_mode(22, pigpio.INPUT)

pi.set_mode(22, pigpio.OUTPUT)
pi.set_mode(9, pigpio.INPUT)
pi.write(22,0)
print('read1: ', pi.read(9))
pi.write(22, 1)
print('read2: ', pi.read(9))
time.sleep(3)
pi.write(22,0)
print('read3: ', pi.read(9))

pi.stop()



#!/usr/bin/env python3
"""
Created on January 9 11:39:15 2016

@author: Alan Yorinks
Copyright (c) 2016 Alan Yorinks All right reserved.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public
License as published by the Free Software Foundation; either
version 3 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

import argparse
import threading
import time

import umsgpack
# noinspection PyPackageRequirements
import zmq
from xideco.data_files.port_map import port_map

import signal
import sys

import Adafruit_BBIO.GPIO as GPIO
import Adafruit_BBIO.PWM as PWM
import Adafruit_BBIO.ADC as ADC


# noinspection PyMethodMayBeStatic,PyUnresolvedReferences,PyUnresolvedReferences,PyUnresolvedReferences
class BeagleBoneBridge:
    """
    The Raspberry Bridge provides the protocol bridge between Xideco and a Raspberry Pi board using the
    pigpio library https://github.com/joan2937/pigpio

    """

    def __init__(self, board_num, board_type, servo_polarity, router_ip_address):
        """
        :param board_num: System Board Number (1-10)
        :param board_type: "black" or "green"
        :param servo_polarity: 1 or 0
        :param: router_ip_address: IP address of xideco router
        :return:
        """
        self.board_num = board_num
        self.board_type = board_type.lower()
        self.servo_polarity = servo_polarity
        self.router_ip_address = router_ip_address

        self.payload = None
        self.black_gpio_pins = ["P9_11", "P9_12", "P9_13", "P9_14", "P9_15", "P9_16", "P9_17", "P9_18",
                                "P9_21", "P9_22", "P9_23", "P9_24", "P9_25", "P9_26", "P9_27", "P9_29",
                                "P9_30", "P9_31", "P9_41", "P9_42", "P8_7", "P8_8", "P8_9", "P8_10",
                                "P8_11", "P8_12", "P8_13", "P8_14", "P8_15", "P8_16", "P8_17", "P8_18",
                                "P8-19", "P8-26"]

        self.green_gpio_pins = ["P9_11", "P9_12", "P9_13", "P9_14", "P9_15", "P9_16", "P9_17", "P9_18",
                                "P9_21", "P9_22", "P9_23", "P9_24", "P9_25", "P9_26", "P9_27", "P9_29",
                                "P9_30", "P9_31", "P9_41", "P9_42", "P8_3", "P8_4", "P8_5", "P8_6",
                                "P8_7", "P8_8", "P8_9", "P8_10", "P8_11", "P8_12", "P8_13", "P8_14",
                                "P8_15", "P8_16", "P8_17", "P8_18", "P8_19", "P8_20", "P8_21", "P8_22",
                                "P8_23", "P8_24", "P8_25", "P8_26", "P8_27", "P8_28", "P8_29", "P8_30",
                                "P8_31", "P8_32", "P8_33", "P8_34", "P8_35", "P8_36", "P8_37", "P8_38",
                                "P8_39", "P8_40", "P8_41", "P8_42", "P8_43", "P8_44" "P8_45", "P8_46"]

        self.i2c_pins = {"scl": "P9_19", "sda": "P9_20"}

        self.analog_pins = ["P9_33", "P9_35", "P9_36", "P9_37", "P9_38", "P9_39", "P9_40"]

        self.pwm_pins = ["P9_14", "P9_16", "P9_21", "P9_22", "P9_42", "P8_13", "P8_19"]

        # this is a list of dictionary items describing a pin
        # each pin dictionary entry contains the pin id, its configured mode, and if it is enabled or not
        self.gpio_pin_states = []
        self.pwm_pin_states = []
        self.analog_pin_states = []

        # set a list called pins to hold the pin modes
        for x in self.black_gpio_pins:
            entry = {'pin': x, 'mode': None, 'enabled': False}
            self.gpio_pin_states.append(entry)

        for x in self.pwm_pins:
            entry = {'pin': x, 'mode': None, 'enabled': False}
            self.pwm_pin_states.append(entry)

        for x in self.analog_pins:
            entry = {'pin': x, 'mode': None, 'enabled': False}
            self.analog_pin_states.append(entry)

        # establish the zeriomq sub and pub sockets
        if self.router_ip_address == 'None':
            self.router_ip_address = port_map.port_map['router_ip_address']
        else:
            self.router_ip_address = router_ip_address

        print('\n**************************************')
        print('BeagleBone Black  Bridge - xibb')
        print('Using router IP address: ' + self.router_ip_address)
        print('**************************************')

        print('\nTo specify some other address for the router, use the -r command line option')

        # establish the zeriomq sub and pub sockets
        self.context = zmq.Context()
        self.subscriber = self.context.socket(zmq.SUB)
        connect_string = "tcp://" + self.router_ip_address + ':' + port_map.port_map['subscribe_to_router_port']
        self.subscriber.connect(connect_string)

        # create the topic we wish to subscribe to
        env_string = "A" + self.board_num
        envelope = env_string.encode()
        self.subscriber.setsockopt(zmq.SUBSCRIBE, envelope)
        # subscribe to broadcast i2c message - i2c messages can also be board specific with A + board number
        self.subscriber.setsockopt(zmq.SUBSCRIBE, 'Q'.encode())

        self.publisher = self.context.socket(zmq.PUB)
        connect_string = "tcp://" + self.router_ip_address + ':' + port_map.port_map['publish_to_router_port']

        self.publisher.connect(connect_string)

        # The Xideco protocol message received
        self.payload = None

        self.command_dict = {'digital_pin_mode': self.setup_digital_pin, 'digital_write': self.digital_write,
                             'analog_pin_mode': self.setup_analog_pin, 'analog_write': self.analog_write,
                             'set_servo_position': self.set_servo_position, 'play_tone': self.play_tone,
                             'tone_off': self.tone_off}

        self.last_problem = ''

        self.analog_reader = None
        self.sonar = None

    def setup_analog_pin(self):
        """
        This method validates and configures a pin for analog input
        :return: None
        """
        # clear out any residual problem strings
        #

        self.last_problem = '2-0\n'
        pin = self.validate_pin(self.analog_pins)
        if pin == 99:
            self.last_problem = '2-1\n'
            return
        index = self.analog_pins.index(pin)
        pin_entry = self.analog_pin_states[index]

        if self.payload['enable'] == 'Enable':
            pin_entry['enabled'] = True
            pin_entry['mode'] = 'analog'
            self.analog_pin_states[index] = pin_entry
        else:
            pin_entry['enabled'] = False
            self.analog_pin_states[index] = pin_entry

        if not self.analog_reader:
            self.analog_reader = AnalogReader(self.board_num, self.analog_pin_states)

            ADC.setup()
            self.analog_reader.start()

    def setup_digital_pin(self):
        """
        This method processes the Scratch "Digital Pin" Block that establishes the mode for the pin
        :return: None
        """

        # clear out any residual problem strings
        # self.report_problem('1-0\n')

        self.last_problem = '1-0\n'

        # retrieve mode from command
        mode = self.payload['mode']

        # Is the user enabling or disabling the pin? Get the 'raw' value and translate it.
        enable = self.payload['enable']

        if mode == 'SONAR':
            self.last_problem = '7-0\n'
            pin = self.validate_pin(self.analog_pins)
            if pin == 99:
                self.last_problem = '7-1\n'
                return
            index = self.analog_pins.index(pin)
            pin_entry = self.analog_pin_states[index]

            if self.payload['enable'] == 'Enable':
                pin_entry['enabled'] = True
                pin_entry['mode'] = 'sonar'
                self.analog_pin_states[index] = pin_entry
            else:
                pin_entry['enabled'] = False
                self.analog_pin_states[index] = pin_entry

            if not self.analog_reader:
                self.analog_reader = AnalogReader(self.board_num, self.analog_pin_states)

                ADC.setup()
                self.analog_reader.start()


        elif mode == 'PWM' or mode == 'Servo' or mode == 'Tone':
            pin = self.validate_pin(self.pwm_pins)
            if pin == 99:
                self.last_problem = '1-2\n'
                return
            index = self.pwm_pins.index(pin)
            pin_entry = self.pwm_pin_states[index]

            if self.payload['enable'] == 'Enable':
                pin_entry['enabled'] = True
                self.pwm_pin_states[index] = pin_entry
                if mode == 'PWM' or mode == 'Tone':
                    PWM.start(pin, 0.0)
                else:
                    PWM.start(pin, 3, 60.0, self.servo_polarity)
            else:
                pin_entry['enabled'] = False
                self.pwm_pin_states[index] = pin_entry
        else:
            pin = self.validate_pin(self.black_gpio_pins)
            if pin == 99:
                self.last_problem = '1-1\n'
                return

            # retrieve mode from command
            mode = self.payload['mode']
            #
            if enable == 'Enable':
                # validate the mode for this pin
                if mode == 'Input':
                    index = self.black_gpio_pins.index(pin)
                    pin_entry = self.gpio_pin_states[index]
                    pin_entry['mode'] = GPIO.IN
                    pin_entry['enabled'] = True

                    self.gpio_pin_states[index] = pin_entry
                    GPIO.setup(pin, GPIO.IN)
                    GPIO.add_event_detect(pin, GPIO.BOTH, callback=self.digital_input_callback)

                elif mode == 'Output':

                    index = self.black_gpio_pins.index(pin)
                    pin_entry = self.gpio_pin_states[index]

                    # update the pin table
                    # pin_entry = {'pin': pin, 'mode': GPIO.OUT, 'enabled': True}

                    pin_entry['mode'] = GPIO.OUT
                    pin_entry['enabled'] = True

                    self.gpio_pin_states[index] = pin_entry
                    GPIO.setup(pin, GPIO.OUT)
            # must be disable
            else:
                if mode == 'Output':
                    index = self.black_gpio_pins.index(pin)
                    pin_entry = self.gpio_pin_states[index]
                    pin_entry['enabled'] = False
                    self.gpio_pin_states[index] = pin_entry
                elif mode == 'Input':
                    index = self.black_gpio_pins.index(pin)
                    pin_entry = self.gpio_pin_states[index]
                    pin_entry['enabled'] = False
                    self.gpio_pin_states[index] = pin_entry
                    GPIO.remove_event_detect(pin)

    def analog_write(self):
        """
        Set a PWM configured pin to the requested value
        :return: None
        """
        # clear out any residual problem strings
        self.last_problem = '4-0\n'

        pin = self.validate_pin(self.pwm_pins)
        if pin == 99:
            self.last_problem = '4-1\n'
            return

        # get pin information
        index = self.pwm_pins.index(pin)
        pin_entry = self.pwm_pin_states[index]

        if not pin_entry['enabled']:
            self.last_problem = '4-2\n'
            return

        value = float(self.payload['value'])

        if not (0 <= value <= 100):
            self.last_problem = '4-3\n'

        PWM.set_duty_cycle(pin, value)

    def digital_write(self):
        """
        Set the state of a digital pin
        :return:
        """
        # clear out any residual problem strings
        self.last_problem = '3-0\n'

        pin = self.validate_pin(self.black_gpio_pins)
        if pin == 99:
            self.last_problem = '3-1\n'
            return

        # get pin information
        index = self.black_gpio_pins.index(pin)
        pin_entry = self.gpio_pin_states[index]
        if pin_entry['mode'] != GPIO.OUT:
            self.last_problem = '3-2\n'
            return

        if not pin_entry['enabled']:
            self.last_problem = '3-3\n'
            return

        value = int(self.payload['value'])
        if value == 0:
            value = GPIO.LOW
        else:
            value = GPIO.HIGH

        GPIO.output(pin, value)

    def play_tone(self):
        """
        This method will play a tone using PWM.
        """

        # clear out any residual problem strings

        self.last_problem = '5-0\n'

        pin = self.validate_pin(self.pwm_pins)
        if pin == 99:
            self.last_problem = '5-1\n'
            return

        # get pin information
        index = self.pwm_pins.index(pin)
        pin_entry = self.pwm_pin_states[index]

        if not pin_entry['enabled']:
            self.last_problem = '5-2\n'
            return

        frequency = int(self.payload['frequency'])
        duration = float(self.payload['duration']) / 1000

        PWM.set_duty_cycle(pin, 50)

        PWM.set_frequency(pin, frequency)

        if duration == 0:
            return

        time.sleep(duration)

        PWM.set_duty_cycle(pin, 0.0)

    def tone_off(self):
        """
        This method forces a pin playing a tone to off
        :return:
        """
        self.last_problem = '6-0\n'

        pin = self.validate_pin(self.pwm_pins)
        if pin == 99:
            self.last_problem = '6-1\n'
            return

        # get pin information
        index = self.pwm_pins.index(pin)
        pin_entry = self.pwm_pin_states[index]

        if not pin_entry['enabled']:
            self.last_problem = '6-2\n'
            return

        PWM.set_duty_cycle(pin, 0.0)

    def set_servo_position(self):

        """
        Set a servo position
        :return:
        """
        duty_min = 3
        duty_max = 14.5
        duty_span = duty_max - duty_min

        self.last_problem = '7-0\n'

        pin = self.validate_pin(self.pwm_pins)
        if pin == 99:
            self.last_problem = '7-1\n'
            return

        index = self.pwm_pins.index(pin)
        pin_entry = self.pwm_pin_states[index]

        if not pin_entry['enabled']:
            self.last_problem = '7-1\n'
            return

        position = int(self.payload['position'])

        angle_f = float(position)
        duty = 100 - ((angle_f / 180) * duty_span + duty_min)

        PWM.set_duty_cycle(pin, duty)

    def validate_pin(self, pin_list):
        """
        Validate a pin in the pin_list
        :param pin_list: pin list specific to the type of pin
        :return: pin number if valid, 99 if invalid
        """
        pin = self.payload['pin'].upper()
        if pin not in pin_list:
            return 99
        else:
            return pin

    def report_problem(self):
        """
        Publish the supplied Xideco protocol message
        :return: None
        """
        # create a topic specific to the board number of this board
        envelope = ("B" + self.board_num).encode()

        msg = umsgpack.packb({u"command": "problem", u"board": 1, u"problem": self.last_problem})

        self.publisher.send_multipart([envelope, msg])
        self.last_problem = ''

    def digital_input_callback(self, pin):
        """
        This method returns a digital pin update by publishing a message
        :param pin: pin
        :return:
        """
        # if the pin has reports disabled, just ignore
        # pin_state = self.pins[gpio]
        state = GPIO.input(pin)

        digital_reply_msg = umsgpack.packb({u"command": "digital_read", u"pin": pin, u"value": str(state)})

        envelope = ("B" + self.board_num).encode()
        self.publisher.send_multipart([envelope, digital_reply_msg])

    def run_bb_bridge(self):
        """
        Start up the bridge
        :return:
        """
        # self.pi.set_mode(11, pigpio.INPUT)
        # cb1 = self.pi.callback(11, pigpio.EITHER_EDGE, self.cbf)
        while True:
            if self.last_problem:
                self.report_problem()
            # noinspection PyBroadException
            try:
                z = self.subscriber.recv_multipart(zmq.NOBLOCK)
                self.payload = umsgpack.unpackb(z[1])
                # print("[%s] %s" % (z[0], self.payload))

                command = self.payload['command']
                if command == 'i2c_request':
                    time.sleep(.001)
                    continue
                elif command in self.command_dict:
                    self.command_dict[command]()
                else:
                    print("can't execute unknown command", str(command))
                    # time.sleep(.001)
            except KeyboardInterrupt:
                self.cleanup()
                sys.exit(0)
            except zmq.error.Again:
                time.sleep(.001)

    def cleanup(self):
        GPIO.cleanup()
        PWM.cleanup()


class AnalogReader(threading.Thread):
    """
    This class handles the pcf8591 YL 40 Module analog to digital conversion module
    """

    def __init__(self, board_num, pin_states):
        """

        :param board_num: board number
        :param pin_states: analog pin state table
        :return: nothing is returned
        """
        super().__init__()

        self.board_num = board_num
        self.pin_states = pin_states

        self.context = zmq.Context()

        self.publisher = self.context.socket(zmq.PUB)
        connect_string = "tcp://" + port_map.port_map['router_ip_address'] + ':' + port_map.port_map[
            'reporter_publisher_port']

        self.publisher.connect(connect_string)

    # noinspection PyMethodMayBeStatic
    def convert_to_distance(self, raw_data):
        distance = None
        value = raw_data / 1000 * 2
        if value > 2.45:
            distance = 10
        if value < .4:
            distance = 80
        elif 2.083 <= value <= 2.45:
            distance = 12
        elif 1.811 <= value <= 2.083:
            distance = 14
        elif 1.620 <= value <= 1.811:
            distance = 16
        elif 1.461 <= value <= 1.620:
            distance = 18
        elif 1.310 <= value <= 1.461:
            distance = 20
        elif 1.211 <= value <= 1.310:
            distance = 22
        elif 1.099 <= value <= 1.211:
            distance = 24
        elif 1.022 <= value <= 1.099:
            distance = 26
        elif .965 <= value <= 1.022:
            distance = 28
        elif .907 <= value <= .965:
            distance = 30
        elif .851 <= value <= .907:
            distance = 32
        elif .8 <= value <= .851:
            distance = 34
        elif .757 <= value <= .8:
            distance = 36
        elif .720 <= value <= .757:
            distance = 38
        elif .695 <= value <= .720:
            distance = 40
        elif .656 <= value <= .695:
            distance = 42
        elif .639 <= value <= .656:
            distance = 44
        elif .612 <= value <= .639:
            distance = 46
        elif .593 <= value <= .612:
            distance = 48
        elif .564 <= value <= .593:
            distance = 50
        elif .543 <= value <= .564:
            distance = 52
        elif .522 <= value <= .543:
            distance = 54
        elif .503 <= value <= .522:
            distance = 56
        elif .483 <= value <= .503:
            distance = 58
        elif .464 <= value <= .483:
            distance = 60
        elif .445 <= value <= .464:
            distance = 62
        elif .447 <= value <= .445:
            distance = 64
        elif .428 <= value <= .447:
            distance = 66
        elif .427 <= value <= .428:
            distance = 68
        elif .413 <= value <= .427:
            distance = 70

        return distance

    def run(self):
        """
        Continuously monitor the A/D
        :return:
        """
        value = None
        while True:
            try:
                for entry in self.pin_states:
                    if entry['enabled']:
                        if entry['mode'] == 'analog':
                            value = ADC.read(entry['pin'])
                            value = round(value, 4)

                        elif entry['mode'] == 'sonar':
                            value = ADC.read_raw(entry['pin'])
                            value = self.convert_to_distance(value)

                        digital_reply_msg = umsgpack.packb({u"command": "analog_read", u"pin": entry['pin'],
                                                            u"value": str(value)})

                        envelope = ("B" + self.board_num).encode()
                        self.publisher.send_multipart([envelope, digital_reply_msg])
                time.sleep(0.05)
            except KeyboardInterrupt:
                sys.exit(0)


def beaglebone_bridge():
    # noinspection PyShadowingNames

    parser = argparse.ArgumentParser()
    parser.add_argument("-b", dest="board_number", default="1", help="Board Number - 1 through 10")
    parser.add_argument("-p", dest="polarity", default="p", help="Servo polarity: p or n")
    parser.add_argument('-r', dest='router_ip_address', default='None', help='Router IP Address')

    # parser.add_argument("-t", dest="board_type", default="black", help="black or green")


    args = parser.parse_args()

    board_num = args.board_number
    board_type = "black"
    servo_polarity = args.polarity
    if servo_polarity == 'p':
        servo_polarity = 1
    else:
        servo_polarity = 0

    router_ip_address = args.router_ip_address

    bb_bridge = BeagleBoneBridge(board_num, board_type, servo_polarity, router_ip_address)

    try:
        bb_bridge.run_bb_bridge()
    except KeyboardInterrupt:
        bb_bridge.cleanup()
        sys.exit(0)

    # signal handler function called when Control-C occurs
    # noinspection PyShadowingNames,PyUnusedLocal,PyUnusedLocal
    def signal_handler(signal, frame):
        print("\nControl-C detected. See you soon.")
        sys.exit(0)

    # listen for SIGINT
    signal.signal(signal.SIGINT, signal_handler)
    signal.signal(signal.SIGTERM, signal_handler)


if __name__ == "__main__":
    beaglebone_bridge()



import zmq
import random
import sys
import time

port = "5556"
context = zmq.Context()
socket = context.socket(zmq.PAIR)
socket.bind("tcp://*:%s" % port)

while True:
    # socket.send_string("Server message to client3")
    [address, contents] = socket.recv_multipart()
    print("[%s] %s" % (address, contents))


    # msg = socket.recv()
    #  print (msg)
    time.sleep(1)


#!/usr/bin/env python3
"""
Created on January 9 11:39:15 2016

@author: Alan Yorinks
Copyright (c) 2016 Alan Yorinks All right reserved.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public
License as published by the Free Software Foundation; either
version 3 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

import argparse
import signal
import sys

import umsgpack
# noinspection PyPackageRequirements
import zmq
from pymata_aio.constants import Constants
from pymata_aio.pymata3 import PyMata3

from xideco.data_files.port_map import port_map


# noinspection PyMethodMayBeStatic,PyUnresolvedReferences,PyUnresolvedReferences,PyUnresolvedReferences
class ArduinoBridge:
    """
    The Arduino Bridge provides the protocol bridge between Xideco and Firmata
    """

    def __init__(self, pymata_board, board_num, router_ip_address):
        """
        :param pymata_board: Pymata-aio instance
        :param board_num: Arduino Board Number (1-10)
        :return:
        """

        self.board_num = board_num
        self.board = pymata_board
        self.router_ip_address = router_ip_address


        # lists of digital pin capabilities
        # These lists contain the pins numbers that support the capability
        self.input_capable = []
        self.output_capable = []
        self.analog_capable = []
        self.pwm_capable = []
        self.servo_capable = []
        self.i2c_capable = []

        # this contains the numeric "A" (A0, A1..) channel values supported by the board
        self.analog_channel = []

        # this is the total number of pins supported by the connected arduino
        self.num_digital_pins = 0

        # for Snap - a dictionary of pins with their latest values
        self.digital_data = {}
        self.analog_data = {}

        # go discover the type of Arduino that we are connected to
        self.get_pin_capabilities()

        # establish the zeriomq sub and pub sockets
        if self.router_ip_address == 'None':
            self.router_ip_address = port_map.port_map['router_ip_address']
        else:
            self.router_ip_address = router_ip_address

        print('\n**************************************')
        print('Arduino Bridge - xiab')
        print('Using router IP address: ' + self.router_ip_address)
        print('**************************************')

        print('\nTo specify some other address for the router, use the -r command line option')

        # establish the zeriomq sub and pub sockets
        self.context = zmq.Context()
        self.subscriber = self.context.socket(zmq.SUB)
        connect_string = "tcp://" + self.router_ip_address + ':' + port_map.port_map['subscribe_to_router_port']
        self.subscriber.connect(connect_string)

        # create the topic we wish to subscribe to
        env_string = "A" + self.board_num
        envelope = env_string.encode()
        self.subscriber.setsockopt(zmq.SUBSCRIBE, envelope)
        # subscribe to broadcast i2c message - i2c messages can also be board specific with A + board number
        self.subscriber.setsockopt(zmq.SUBSCRIBE, 'Q'.encode())

        self.publisher = self.context.socket(zmq.PUB)
        connect_string = "tcp://" + self.router_ip_address + ':' + port_map.port_map['publish_to_router_port']

        self.publisher.connect(connect_string)

        # The Xideco protocol message received
        self.payload = None

        # "pointers" to the methods to process commands from the user
        self.command_dict = {'digital_pin_mode': self.setup_digital_pin, 'digital_write': self.digital_write,
                             'analog_pin_mode': self.setup_analog_pin, 'analog_write': self.analog_write,
                             'set_servo_position': self.set_servo_position, 'play_tone': self.play_tone,
                             'tone_off': self.tone_off, 'i2c_request': self.i2c_request}

        self.last_problem = ''

        self.i2c_report_pending = False

    def setup_analog_pin(self):
        """
        This method validates and configures a pin for analog input
        :return: None
        """
        # clear out any residual problem strings
        self.last_problem = '2-0\n'
        self.report_problem()

        try:
            pin = int(self.payload['pin'])
        except ValueError:
            self.last_problem = '2-1\n'
            return

        # validate that pin in the analog channel list
        # pin numbers start with 0
        if pin not in self.analog_channel:
            self.last_problem = '2-2\n'
            return

        # Is the user enabling or disabling the pin? Get the 'raw' value and
        # translate it.
        enable = self.payload['enable']

        if enable == 'Enable':
            self.board.set_pin_mode(pin, Constants.ANALOG, self.analog_input_callback)
        else:
            self.board.disable_analog_reporting(pin)

    def setup_digital_pin(self):
        """
        This method processes the Scratch "Digital Pin" Block that establishes the mode for the pin
        :return: None
        """

        # clear out any residual problem strings
        self.last_problem = '1-0\n'

        # convert pin string to integer.
        try:
            pin = int(self.payload['pin'])
        except ValueError:
            # Pin Must Be Specified as an Integer 1-1
            self.last_problem = '1-1\n'
            return

        # validate that the pin is within the pin count range
        # pin numbers start with 0 1-2
        if pin >= self.num_digital_pins:
            self.last_problem = '1-2\n'
            return

        # Is the user enabling or disabling the pin? Get the 'raw' value and translate it.
        enable = self.payload['enable']

        # retrieve mode from command
        mode = self.payload['mode']

        if enable == 'Enable':
            # validate the mode for this pin
            if mode == 'Input':
                if pin in self.input_capable:
                    # send the pin mode to the arduino
                    self.board.set_pin_mode(pin, Constants.INPUT, self.digital_input_callback)
                else:
                    # this pin does not support input mode
                    self.last_problem = '1-3\n'
            # elif mode == 'Output':
            elif mode == 'Output':
                if pin in self.output_capable:
                    # send the pin mode to the arduino
                    self.board.set_pin_mode(int(pin), Constants.OUTPUT)
                else:
                    # this pin does not support output mode
                    self.last_problem = '1-4\n'

            elif mode == 'PWM':
                if pin in self.pwm_capable:
                    # send the pin mode to the arduino
                    self.board.set_pin_mode(pin, Constants.PWM)
                else:
                    # this pin does not support output mode
                    self.last_problem = '1-5\n'
            elif mode == 'Servo':
                if pin in self.servo_capable:
                    # send the pin mode to the arduino
                    self.board.set_pin_mode(pin, Constants.SERVO)
                else:
                    # this pin does not support output mode
                    self.last_problem = '1-6\n'
            elif mode == 'Tone':
                if pin in self.servo_capable:
                    # send the pin mode to the arduino
                    self.board.set_pin_mode(pin, Constants.OUTPUT)
                else:
                    # this pin does not support output mode
                    self.last_problem = '1-7\n'
            elif mode == 'SONAR':
                if pin in self.input_capable:
                    # send the pin mode to the arduino
                    self.board.sonar_config(pin, pin, self.digital_input_callback, Constants.CB_TYPE_ASYNCIO)
                else:
                    # this pin does not support output mode
                    self.last_problem = '1-8\n'
            else:
                self.last_problem = '1-9\n'
        # must be disable
        else:
            pin_state = self.board.get_pin_state(pin)
            if pin_state[1] != Constants.INPUT:
                self.last_problem = '1-10\n'
            else:
                # disable the pin
                self.board.disable_digital_reporting(pin)

    def analog_write(self):
        """
        Set a PWM configured pin to the requested value
        :return: None
        """

        self.last_problem = '4-0\n'
        # clear out any residual problem strings

        try:
            pin = int(self.payload['pin'])
        except ValueError:
            # Pin Must Be Specified as an Integer
            self.last_problem = '4-1\n'
            return

        pin_state = self.board.get_pin_state(pin)
        if len(pin_state) == 1:
            self.last_problem = '4-2\n'
            return

        if pin_state[1] != Constants.PWM:
            self.last_problem = '4-3\n'
            return

        try:
            value = int(self.payload['value'])
        except ValueError:
            # Pin Must Be Specified as an Integer
            self.last_problem = '4-4\n'
            return

        # validate range of value
        if 0 <= value <= 255:
            self.board.analog_write(pin, value)
        else:
            self.last_problem = '4-5\n'

    def digital_write(self):
        """
        Set the state of a digital pin
        :return:
        """

        # clear out any residual problem strings
        self.last_problem = '3-0\n'

        try:
            pin = int(self.payload['pin'])
        except ValueError:
            self.last_problem = '3-1\n'
            return

        pin_state = self.board.get_pin_state(pin)
        if len(pin_state) == 1:
            self.last_problem = '3-2\n'
            return

        if pin_state[1] != Constants.OUTPUT:
            self.last_problem = '3-3\n'
            return

        value = int(self.payload['value'])
        self.board.digital_write(pin, value)

    def play_tone(self):
        """
        This method will play a tone using the Arduino tone library. It requires FirmataPlus
        :return: None
        """
        # clear out any residual problem strings
        self.last_problem = '5-0\n'
        # get the pin string from the block
        try:
            pin = int(self.payload['pin'])
        except ValueError:
            # Pin Must Be Specified as an Integer
            self.last_problem = '5-1\n'
            return

        pin_state = self.board.get_pin_state(pin)
        if len(pin_state) == 1:
            self.last_problem = '5-2\n'
            return

        if pin_state[1] != Constants.OUTPUT:
            self.last_problem = '5-3\n'
            return

        frequency = self.payload['frequency']

        try:
            frequency = int(frequency)
        except ValueError:
            # frequency Must Be Specified as an Integer
            self.last_problem = '5-4\n'
            return

        try:
            duration = int(self.payload['duration'])
        except ValueError:
            # frequency Must Be Specified as an Integer
            self.last_problem = '5-5\n'
            return

        self.board.play_tone(pin, Constants.TONE_TONE, frequency, duration)

    def tone_off(self):
        """
        Turn tone off
        :return:
        """
        # clear out any residual problem strings
        self.last_problem = '6-0\n'
        # get the pin string from the block
        try:
            pin = int(self.payload['pin'])
        except ValueError:
            # Pin Must Be Specified as an Integer
            self.last_problem = '6-1\n'
            return

        pin_state = self.board.get_pin_state(pin)
        if len(pin_state) == 1:
            self.last_problem = '6-2\n'
            return

        if pin_state[1] != Constants.OUTPUT:
            self.last_problem = '6-3\n'
            return

        self.board.play_tone(pin, Constants.TONE_NO_TONE, None, None)
        return

    def set_servo_position(self):
        """
        Set a servo position
        :return:
        """
        # clear out any residual problem strings
        self.last_problem = '7-0\n'

        try:
            pin = int(self.payload['pin'])
        except ValueError:
            self.last_problem = '7-1\n'
            return

        pin_state = self.board.get_pin_state(pin)
        if len(pin_state) == 1:
            self.last_problem = '7-2\n'
            return

        if pin_state[1] != Constants.SERVO:
            self.last_problem = '7-3\n'
            return

        position = self.payload['position']

        try:
            position = int(position)
        except ValueError:
            # frequency Must Be Specified as an Integer
            self.last_problem = '7-4\n'
            return

        if 0 <= position <= 180:
            self.board.analog_write(pin, position)
        else:
            self.last_problem = '7-5\n'
        return

    def digital_input_callback(self, data):
        """
        This method receives digital data inputs, creates a Xideco protocol publishing message and publishes the message
        :param data: data[0] = pin, data[1] = value
        :return: None
        """
        pin = str(data[0])
        value = str(data[1])

        digital_reply_msg = umsgpack.packb({u"command": "digital_read", u"pin": pin, u"value": value})

        envelope = ("B" + self.board_num).encode()
        self.publisher.send_multipart([envelope, digital_reply_msg])
        # print(digital_reply_msg)
        # print(envelope)

    def analog_input_callback(self, data):
        """
        This method receives Analog data inputs, creates a Xideco protocol publishing message and publishes the message
        :param data: data[0] = pin, data[1] = value
        :return: None
        """
        pin = str(data[0])
        value = str(data[1])

        analog_reply_msg = umsgpack.packb({u"command": "analog_read", u"pin": pin, u"value": value})

        envelope = ("B" + self.board_num).encode()
        self.publisher.send_multipart([envelope, analog_reply_msg])

    def i2c_request(self):
        """
        This method parses the i2c request and translates it to a native request
        :return:
        """
        while self.i2c_report_pending:
            self.board.sleep(.001)
        cmd = self.payload['cmd']
        addr = self.payload['device_address']

        if cmd == 'init':
            self.board.i2c_config()
        elif cmd == 'write_byte':
            # get handle
            value = self.payload['value']
            register = self.payload['register']
            self.board.i2c_write_request(addr, [register, value])
        elif cmd == 'read_block':
            num_bytes = self.payload['num_bytes']
            register = self.payload['register']
            self.i2c_report_pending = True

            self.board.i2c_read_request(addr, register, num_bytes, Constants.I2C_READ, self.report_i2c_data)
            self.board.sleep(.001)
        else:
            print('unknown cmd')

    def report_i2c_data(self, data):
        # create a topic specific to the board number of this board
        envelope = ("B" + self.board_num).encode()

        msg = umsgpack.packb({u"command": "i2c_reply", u"board": self.board_num, u"data": data[2:]})

        self.publisher.send_multipart([envelope, msg])
        self.i2c_report_pending = False


    def run_arduino_bridge(self):
        """
        start the bridge
        :return:
        """

        while True:
            if self.last_problem:
                self.report_problem()
            # noinspection PyBroadException
            try:
                z = self.subscriber.recv_multipart(zmq.NOBLOCK)

                self.payload = umsgpack.unpackb(z[1])
                # print("[%s] %s" % (z[0], self.payload))
                command = self.payload['command']
                if command in self.command_dict:
                    self.command_dict[command]()
                else:
                    print("can't execute unknown command'")
                self.board.sleep(.001)
            except zmq.error.Again:
                self.board.sleep(.001)
            # return

    def get_pin_capabilities(self):
        """
        This method retrieves the Arduino pin capability and analog map reports.
        For each digital pin mode, a list of valid pins is constructed,
        A total pin count is calculated and in addition,
        a list of valid analog input channels is constructed.
        :return: None
        """
        # get the capability report
        pin_capabilities = self.board.get_capability_report()

        # initialize the total pin count to o
        pin_count = 0

        pin_data = []

        # Each set of digital pin capabilities is delimited by the value of 127
        # Accumulate all of the capabilities into a list for the current pin
        for x in pin_capabilities:
            if x != 127:
                pin_data.append(x)
                continue
            # Found a delimiter, populate the specific capability lists with this pin.
            # Each capability contains 2 bytes. The first is the capability and the second is the
            # number of bits of data resolution for the pin. The resolution is ignored
            else:
                pin__x_capabilities = pin_data[::2]
                for y in pin__x_capabilities:
                    if y == 0:
                        self.input_capable.append(pin_count)
                    elif y == 1:
                        self.output_capable.append(pin_count)
                    elif y == 2:
                        self.analog_capable.append(pin_count)
                    elif y == 3:
                        self.pwm_capable.append(pin_count)
                    elif y == 4:
                        self.servo_capable.append(pin_count)
                    elif y == 6:
                        self.i2c_capable.append(pin_count)
                    elif 7 < y < 14:
                        pass
                    else:
                        print('Unknown Pin Type ' + str(y))
                # clear the pin_data list for the next pin and bump up the pin count
                pin_data = []
                # add an entry into the digital data dictionary
                self.digital_data[pin_count] = 0

                pin_count += 1
        # Done with digital pin discovery, save the pin count
        self.num_digital_pins = pin_count

        # Get analog channel data and create the analog_channel list
        analog_pins = self.board.get_analog_map()
        for x in analog_pins:
            if x != 127:
                self.analog_channel.append(x)
                self.analog_data[x] = 0

    # def report_problem(self, problem):
    def report_problem(self):

        """
        Publish the supplied Xideco protocol message
        :return:
        """
        # create a topic specific to the board number of this board
        # envelope = ("B" + self.board_num).encode()
        # self.publisher.send_multipart([envelope, problem])

        envelope = ("B" + self.board_num).encode()

        msg = umsgpack.packb({u"command": "problem", u"board": 1, u"problem": self.last_problem})

        self.publisher.send_multipart([envelope, msg])
        self.last_problem = ''

    def clean_up(self):
        """
        Clean things up on exit
        :return:
        """
        self.subscriber.close()
        self.publisher.close()
        self.context.term()


def arduino_bridge():
    """
    Main function for arduino bridge
    :return:
    """
    # noinspection PyShadowingNames

    parser = argparse.ArgumentParser()
    parser.add_argument("-b", dest="board_number", default="1", help="Board Number - 1 through 10")
    parser.add_argument("-p", dest="comport", default="None", help="Arduino COM port - e.g. /dev/ttyACMO or COM3")
    parser.add_argument('-r', dest='router_ip_address', default='None', help='Router IP Address')


    args = parser.parse_args()
    if args.comport == "None":
        pymata_board = PyMata3()
    else:
        pymata_board = PyMata3(com_port=args.comport)

    board_num = args.board_number

    router_ip_address = args.router_ip_address

    abridge = ArduinoBridge(pymata_board, board_num, router_ip_address)
    # while True:
    abridge.run_arduino_bridge()

    # signal handler function called when Control-C occurs
    # noinspection PyShadowingNames,PyUnusedLocal,PyUnusedLocal
    def signal_handler(signal, frame):
        print("Control-C detected. See you soon.")

        abridge.clean_up()

        sys.exit(0)

    # listen for SIGINT
    signal.signal(signal.SIGINT, signal_handler)
    signal.signal(signal.SIGTERM, signal_handler)


if __name__ == "__main__":

    try:
        arduino_bridge()
    except KeyboardInterrupt:
        sys.exit(0)



"""

   Pubsub envelope publisher

   Author: Guillaume Aubert (gaubert) <guillaume(dot)aubert(at)gmail(dot)com>

"""
import time
import zmq

def main():
    """main method"""

    # Prepare our context and publisher
    context   = zmq.Context()
    publisher = context.socket(zmq.PUB)
    publisher.bind("tcp://*:5563")

    while True:
        # Write two messages, each with an envelope and content
        publisher.send_multipart([b"A", b"We don't want to see this"])
        publisher.send_multipart([b"B", b"We would like to see this"])
        time.sleep(1)

    # We never get here but clean up anyhow
    publisher.close()
    context.term()


if __name__ == "__main__":
    main()



#!/usr/bin/env python3

import time

import pigpio

class ranger:
   """
   This class encapsulates a type of acoustic ranger.  In particular
   the type of ranger with separate trigger and echo pins.

   A pulse on the trigger initiates the sonar ping and shortly
   afterwards a sonar pulse is transmitted and the echo pin
   goes high.  The echo pins stays high until a sonar echo is
   received (or the response times-out).  The time between
   the high and low edges indicates the sonar round trip time.
   """

   def __init__(self, pi, trigger, echo):
      """
      The class is instantiated with the Pi to use and the
      gpios connected to the trigger and echo pins.
      """
      self.pi    = pi
      self._trig = trigger
      # self._trig = 22
      self._echo = echo
      # self._echo = 9

      self._ping = False
      self._high = None
      self._time = None

      self._triggered = False

      self._trig_mode = pi.get_mode(self._trig)
      self._echo_mode = pi.get_mode(self._echo)

      pi.set_mode(self._trig, pigpio.OUTPUT)
      pi.set_mode(self._echo, pigpio.INPUT)

      self._cb = pi.callback(self._trig, pigpio.EITHER_EDGE, self._cbf)
      self._cb = pi.callback(self._echo, pigpio.EITHER_EDGE, self._cbf)

      self._inited = True


   def _cbf(self, gpio, level, tick):
      if gpio == self._trig:
         if level == 0: # trigger sent
            self._triggered = True
            self._high = None
      else:
         if self._triggered:
            if level == 1:
               self._high = tick
            else:
               if self._high is not None:
                  self._time = tick - self._high
                  self._high = None
                  self._ping = True

   def read(self):
      """
      Triggers a reading.  The returned reading is the number
      of microseconds for the sonar round-trip.

      round trip cms = round trip time / 1000000.0 * 34030
      """
      if self._inited:
         self._ping = False
         self.pi.gpio_trigger(self._trig)
         start = time.time()
         while not self._ping:
            if (time.time()-start) > 5.0:
               return 20000
            time.sleep(0.001)
         return self._time
      else:
         return None

   def cancel(self):
      """
      Cancels the ranger and returns the gpios to their
      original mode.
      """
      if self._inited:
         self._inited = False
         self._cb.cancel()
         self.pi.set_mode(self._trig, self._trig_mode)
         self.pi.set_mode(self._echo, self._echo_mode)

if __name__ == "__main__":

   import time

   import pigpio

   import sonar_trigger_echo

   pi = pigpio.pi()

   sonar = sonar_trigger_echo.ranger(pi, 4, 17)

   end = time.time() + 600.0

   r = 1
   while time.time() < end:

      x = sonar.read()
      x = x / 1000000.0 * 34030.0
      x = x / 2
      x = round(x, 2)
      # print(sonar.read())
      # print("{} {}".format(r, sonar.read()))
      print("{} {}".format(r, x))

      r += 1
      time.sleep(0.03)

   sonar.cancel()

   pi.stop()


#!/usr/bin/env python3
"""
Created on January 9 11:39:15 2016

@author: Alan Yorinks
Copyright (c) 2016 Alan Yorinks All right reserved.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public
License as published by the Free Software Foundation; either
version 3 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""
import os
import signal
import socket
import sys
import time

import zmq

from xideco.data_files.port_map import port_map


# noinspection PyUnresolvedReferences,PyUnresolvedReferences,PyUnresolvedReferences,PyUnresolvedReferences,PyUnresolvedReferences
class XidecoRouter:
    """
    This class consists of a PAIR connection to a control program bridge (i.e. - HTTP for Scratch),
    creates a publisher for Scratch commands, and creates a set of subscribers to listen
    for board data changes.
    """

    def __init__(self):
        """
        This is the constructor for the XidecoRouter class.
        :param: use_port_map: If true, use the ip address in the port map, if false, use discovered ip address
        :return: None
        """
        # figure out the IP address of the router
        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        # use the google dns
        s.connect(('8.8.8.8', 0))
        self.ip_addr = s.getsockname()[0]

        # identify the router ip address for the user on the console
        print('\nXideco Router - xirt')

        print('\n******************************************')
        print('Using router IP address = ' + self.ip_addr)
        print('******************************************')



        # find the path to the data files needed for operation
        path = sys.path

        self.base_path = None

        # get the prefix
        prefix = sys.prefix
        for p in path:
            # make sure the prefix is in the path to avoid false positives
            if prefix in p:
                # look for the configuration directory
                s_path = p + '/xideco/data_files/configuration'
                if os.path.isdir(s_path):
                    # found it, set the base path
                    self.base_path = p + '/xideco'
                    break

        if not self.base_path:
            print('Cannot locate xideco configuration directory.')
            sys.exit(0)

        print('\nIf using the port map, port_map.py is located at:\n')
        print(self.base_path + '/data_files/port_map\n')

        print('NOTE: The path to port_map.py may be different')
        print('for each Operating System/Computer.')

        print('\nSet the router_ip_address entry in port_map.py ')
        print('to the address printed above for each ')
        print('computer running Xideco, or optionally ')
        print('set the address manually for each Xideco module')
        print('using the command line options.\n')

        self.router = zmq.Context()
        # establish router as a ZMQ FORWARDER Device

        # subscribe to any message that any entity publishes
        self.publish_to_router = self.router.socket(zmq.SUB)
        bind_string = 'tcp://' + self.ip_addr + ':' + port_map.port_map[
            'publish_to_router_port']
        self.publish_to_router.bind(bind_string)
        # Don't filter any incoming messages, just pass them through
        self.publish_to_router.setsockopt_string(zmq.SUBSCRIBE, '')

        # publish these messages
        self.subscribe_to_router = self.router.socket(zmq.PUB)
        bind_string = 'tcp://' + self.ip_addr + ':' + port_map.port_map[
            'subscribe_to_router_port']
        self.subscribe_to_router.bind(bind_string)

        zmq.device(zmq.FORWARDER, self.publish_to_router, self.subscribe_to_router)

    # noinspection PyMethodMayBeStatic
    def route(self):
        """
        This method runs in a forever loop.
        :return:
        """
        while True:
            try:
                time.sleep(.001)
            except KeyboardInterrupt:
                sys.exit(0)

    def clean_up(self):
        self.publish_to_router.close()
        self.subscribe_to_router.close()
        self.router.term()


def xideco_router():
    # noinspection PyShadowingNames

    xideco_router = XidecoRouter()
    xideco_router.route()

    # signal handler function called when Control-C occurs
    # noinspection PyShadowingNames,PyUnusedLocal,PyUnusedLocal
    def signal_handler(signal, frame):
        print('Control-C detected. See you soon.')

        xideco_router.clean_up()

        sys.exit(0)

    # listen for SIGINT
    signal.signal(signal.SIGINT, signal_handler)
    signal.signal(signal.SIGTERM, signal_handler)


# Instantiate the router and start the route loop
if __name__ == '__main__':
    xideco_router()



#!/usr/bin/env python3
"""
Created on January 2 11:39:15 2016

@author: Alan Yorinks
Copyright (c) 2016 Alan Yorinks All right reserved.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public
License as published by the Free Software Foundation; either
version 3 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

import asyncio
import configparser
import os
import signal
import sys
import umsgpack

from aiohttp import web
# noinspection PyPackageRequirements
import zmq
from xideco.data_files.port_map import port_map


class HttpBridge:
    """
    This is an HTTP bridge that translates Scratch HTTP requests into xideco protocol messages
   """

    def __init__(self):
        """
        This is the constructor for the xideco HTTP bridge
        :return:
        """

        # find the path to the data files needed for operation
        path = sys.path

        self.base_path = None

        # get the prefix
        prefix = sys.prefix
        for p in path:
            # make sure the prefix is in the path to avoid false positives
            if prefix in p:
                # look for the configuration directory
                s_path = p + '/xideco/data_files/configuration'
                if os.path.isdir(s_path):
                    # found it, set the base path
                    self.base_path = p + '/xideco'
                    break

        if not self.base_path:
            print('Cannot locate xideco configuration directory.')
            sys.exit(0)

        print('\nScratch Project Files Located at:')
        print(self.base_path + '/data_files/scratch_files/projects\n')
        print('port_map.py is located at:')
        print(self.base_path + '/data_files/port_map\n')

        # Scratch poll reply string
        self.poll_reply = ""

        # grab the config file and get it ready for parsing
        config = configparser.ConfigParser()
        config_file_path = str(self.base_path + '/data_files/configuration/configuration.cfg')
        config.read(config_file_path, encoding="utf8")

        # parse the file and place the translation information into the appropriate variable
        self.ln_languages = config.get('translation_lists', 'ln_languages').split(',')
        self.ln_ENABLE = config.get('translation_lists', 'ln_ENABLE').split(',')
        self.ln_DISABLE = config.get('translation_lists', 'ln_DISABLE').split(',')
        self.ln_INPUT = config.get('translation_lists', 'ln_INPUT').split(',')
        self.ln_OUTPUT = config.get('translation_lists', 'ln_OUTPUT').split(',')
        self.ln_PWM = config.get('translation_lists', 'ln_PWM').split(',')
        self.ln_SERVO = config.get('translation_lists', 'ln_SERVO').split(',')
        self.ln_TONE = config.get('translation_lists', 'ln_TONE').split(',')
        self.ln_SONAR = config.get('translation_lists', 'ln_SONAR').split(',')
        self.ln_OFF = config.get('translation_lists', 'ln_OFF').split(',')
        self.ln_ON = config.get('translation_lists', 'ln_ON').split(',')

    # noinspection PyShadowingNames,PyAttributeOutsideInit,PyAttributeOutsideInit
    async def init(self, loop):
        """
        This method initializes the aiohttp server.
        It also instantiate all of the s2aio servers specified in the configuration file.
        After the servers are initialized, the "poll" command is added to the aiohttp server, so that the
        "green" Scratch connectivity indicator stays red until after the servers are instantiated.
        :param loop: asyncio event loop
        :return: http server instance
        """

        app = web.Application(loop=loop)

        app.router.add_route('GET', '/digital_pin_mode/{board}/{enable}/{pin}/{mode}', self.setup_digital_pin)
        app.router.add_route('GET', '/analog_pin_mode/{board}/{enable}/{pin}', self.setup_analog_pin)
        app.router.add_route('GET', '/digital_write/{board}/{pin}/{value}', self.digital_write)
        app.router.add_route('GET', '/analog_write/{board}/{pin}/{value}', self.analog_write)
        app.router.add_route('Get', '/analog_read/{board}/{pin}/{value}', self.got_analog_report)
        app.router.add_route('Get', '/digital_read/{board}/{pin}/{value}', self.got_digital_report)
        app.router.add_route('GET', '/play_tone/{board}/{pin}/{frequency}/{duration}', self.play_tone)

        app.router.add_route('Get', '/problem/{board}/{problem}', self.got_problem_report)
        app.router.add_route('GET', '/set_servo_position/{board}/{pin}/{position}', self.set_servo_position)
        app.router.add_route('GET', '/tone_off/{board}/{pin}', self.tone_off)
        app.router.add_route('GET', '/tweet/{message}', self.tweet)

        srv = await loop.create_server(app.make_handler(), '127.0.0.1', 50208)
        self.loop = loop
        # create a zeromq pair client
        context = zmq.Context()

        self.router_socket = context.socket(zmq.PAIR)
        connect_string = "tcp://" + port_map.port_map['router_ip_address'] + ':' + port_map.port_map['http_port']

        self.router_socket.connect(connect_string)

        app.router.add_route('GET', '/poll', self.poll)
        await self.keep_alive()

        return srv

    async def tweet(self, request):
        """
        Create a tweet command message
        :param request: command and data from GET request
        :return: HTTP response
        """
        # retrieve the users tweet text  from the GET request
        message = request.match_info.get('message')

        # print the message for debug purposes
        print(message)

        # create a msgpack message for the text
        command_msg = umsgpack.packb({u"message": message})

        # Use the pseudo board 100 for message envelope and send the message to the router
        await self.send_command_to_router("100", command_msg)
        return web.Response(body="ok".encode('utf-8'))

    async def setup_digital_pin(self, request):
        """
        This method handles the "set digital pin mode" block request
        :param request: HTTP request
        :return: HTTP response
        """
        command = "digital_pin_mode"
        board = request.match_info.get('board')
        enable = request.match_info.get('enable')
        enable = await self.check_cmd_enable_disable(enable)

        pin = request.match_info.get('pin')
        mode = request.match_info.get('mode')

        mode = await self.check_cmd_digital_mode(mode)
        command_msg = umsgpack.packb({u"command": command, u"enable": enable, u"pin": pin, u"mode": mode})

        await self.send_command_to_router(board, command_msg)

        return web.Response(body="ok".encode('utf-8'))

    async def setup_analog_pin(self, request):
        """
        This method handles the "set analog input pin mode"
        :param request: HTTP request
        :return: HTTP response
        """
        command = "analog_pin_mode"
        board = request.match_info.get('board')
        enable = request.match_info.get('enable')
        enable = await self.check_cmd_enable_disable(enable)

        pin = request.match_info.get('pin')
        command_msg = umsgpack.packb({u"command": command, u"enable": enable, u"pin": pin})
        # await self.send_command_to_router(board, command_msg)

        board = 'A' + board
        board = board.encode()
        self.router_socket.send_multipart([board, command_msg])

        return web.Response(body="ok".encode('utf-8'))

    async def digital_write(self, request):
        """
        This method handles the digital write request
        :param request: HTTP request
        :return: HTTP response
        """
        command = "digital_write"
        board = request.match_info.get('board')
        pin = request.match_info.get('pin')
        value = request.match_info.get('value')
        command_msg = umsgpack.packb({u"command": command, u"pin": pin, u"value": value})
        await self.send_command_to_router(board, command_msg)

        return web.Response(body="ok".encode('utf-8'))

    async def analog_write(self, request):
        """
        This method handles the analog (PWM) write request
        :param request: HTTP request
        :return: HTTP response
        """
        command = "analog_write"
        board = request.match_info.get('board')
        pin = request.match_info.get('pin')
        value = request.match_info.get('value')
        command_msg = umsgpack.packb({u"command": command, u"pin": pin, u"value": value})
        await self.send_command_to_router(board, command_msg)

        return web.Response(body="ok".encode('utf-8'))

    async def play_tone(self, request):
        """
        This method handles the play tone request.
        :param request: HTTP request
        :return: HTTP response
        """
        command = 'play_tone'
        board = request.match_info.get('board')
        pin = request.match_info.get('pin')
        freq = request.match_info.get('frequency')
        duration = request.match_info.get('duration')
        command_msg = umsgpack.packb({u"command": command, u"pin": pin, u"frequency": freq, u"duration": duration})
        await self.send_command_to_router(board, command_msg)

        return web.Response(body="ok".encode('utf-8'))

    async def tone_off(self, request):
        """
        This method turns tone off.
        :param request: HTTP request
        :return: HTTP response
        """
        command = 'tone_off'
        board = request.match_info.get('board')
        pin = request.match_info.get('pin')

        command_msg = umsgpack.packb({u"command": command, u"pin": pin})
        await self.send_command_to_router(board, command_msg)

        return web.Response(body="ok".encode('utf-8'))

    async def set_servo_position(self, request):
        """
        This method sets a servo position.
        :param request: HTTP request
        :return: HTTP response
        """
        command = 'set_servo_position'
        board = request.match_info.get('board')
        pin = request.match_info.get('pin')
        position = request.match_info.get('position')

        command_msg = umsgpack.packb({u"command": command, u"pin": pin, u"position": position})
        await self.send_command_to_router(board, command_msg)

        return web.Response(body="ok".encode('utf-8'))

    # noinspection PyUnusedLocal
    async def poll(self, request):
        """
        This method handles the Scratch poll request for reporter data
        :param request: HTTP request
        :return: HTTP response
        """
        # save the reply to a temporary variable
        total_reply = self.poll_reply
        if total_reply != '':
            print('r: ' + total_reply)

        # clear the poll reply string for the next reply set
        self.poll_reply = ""
        return web.Response(headers={"Access-Control-Allow-Origin": "*"},
                            content_type="text/html", charset="ISO-8859-1", text=total_reply)

    async def got_analog_report(self, request):
        """
        This method handles analog data reports being sent from s2aio servers
        :param request: HTTP request
        :return: HTTP response
        """
        board = request.match_info.get('board')
        pin = request.match_info.get('pin')
        value = request.match_info.get('value')
        self.poll_reply += 'analog_read/' + board + '/' + pin + ' ' + value + '\n'

        return web.Response(body="ok".encode('utf-8'))

    async def got_digital_report(self, request):
        """
        This method handles data data reports being sent from s2aio servers
        :param request: HTTP request
        :return: HTTP response
        """
        board = request.match_info.get('board')
        pin = request.match_info.get('pin')
        value = request.match_info.get('value')
        self.poll_reply += 'digital_read/' + board + '/' + pin + ' ' + value + '\n'

        return web.Response(body="ok".encode('utf-8'))

    async def got_problem_report(self, request):
        """
        This method handles problem (debugging) reports being sent from s2aio servers
        :param request: HTTP request
        :return: HTTP response
        """
        board = request.match_info.get('board')
        problem = request.match_info.get('problem')
        self.poll_reply += 'problem/' + board + ' ' + problem + '\n'
        return web.Response(body="ok".encode('utf-8'))

    async def send_command_to_router(self, board, message):
        """
        Create a topic message and send the multipart message to the router
        :param board: Arduino board message is destined for
        :param message: Command message from Scratch
        :return:
        """
        m_topic = 'A' + board
        topic = m_topic.encode()
        self.router_socket.send_multipart([topic, message])

    async def keep_alive(self):
        """
        This method is used to keep the server up and running when not connected to Scratch
        :return:
        """
        while True:

            # check for reporter messages
            try:
                [address, contents] = self.router_socket.recv_multipart(zmq.NOBLOCK)
                payload = umsgpack.unpackb(contents)
                # print("[%s] %s" % (address, payload))
                board_num = address.decode()
                board_num = board_num[1]
                command = payload['command']
                if command == 'problem':
                    data_string = command + '/' + board_num + ' ' + payload['problem']
                else:
                    pin = payload['pin']
                    value = payload['value']
                    data_string = command + '/' + board_num + '/' + pin + ' ' + value + '\n'
                # print(data_string)
                self.poll_reply += data_string

            except zmq.error.Again:
                pass
            await asyncio.sleep(.001)

    async def check_cmd_enable_disable(self, command):
        """
        This method provides translation for enable/disable
        :param command: Language specific value for enable
        :return: English translation
        """
        if command in self.ln_ENABLE:
            return 'Enable'
        elif command in self.ln_DISABLE:
            return 'Disable'
        else:
            return 'invalid'

    # noinspection PyPep8Naming
    async def check_cmd_digital_mode(self, command):
        """
        This method provides translation for the digital mode.
        :param command: Mode in native language
        :return: Mode in english
        """
        if command in self.ln_INPUT:
            return 'Input'
        if command in self.ln_OUTPUT:
            return 'Output'
        if command in self.ln_PWM:
            return 'PWM'
        if command in self.ln_SERVO:
            return 'Servo'
        if command in self.ln_TONE:
            return 'Tone'
        if command in self.ln_SONAR:
            return 'SONAR'


def http_bridge():
    # noinspection PyShadowingNames

    http_bridge = HttpBridge()
    # noinspection PyShadowingNames
    loop = asyncio.get_event_loop()

    # noinspection PyBroadException
    try:
        loop.run_until_complete(http_bridge.init(loop))
    except:
        # noinspection PyShadowingNames
        loop = asyncio.get_event_loop()

        sys.exit(0)

    # signal handler function called when Control-C occurs
    # noinspection PyShadowingNames,PyUnusedLocal,PyUnusedLocal
    def signal_handler(signal, frame):
        print("Control-C detected. See you soon.")

        for t in asyncio.Task.all_tasks(loop):
            # noinspection PyBroadException
            try:
                t.cancel()
                loop.run_until_complete(asyncio.sleep(.1))
                loop.stop()
                loop.close()
            except:
                pass

        sys.exit(0)

    # listen for SIGINT
    signal.signal(signal.SIGINT, signal_handler)
    signal.signal(signal.SIGTERM, signal_handler)


if __name__ == "__main__":

    try:
        http_bridge()
    except KeyboardInterrupt:
        sys.exit(0)

    loop = asyncio.get_event_loop()

    # noinspection PyBroadException
    try:
        loop.run_forever()
        loop.stop()
        loop.close()
    except:
        pass



"""
Copyright (c) 2016 Alan Yorinks All right reserved.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public
License as published by the Free Software Foundation; either
version 3 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

import socket
import sys
import time

import umsgpack
import zmq


# noinspection PyUnresolvedReferences
class XideKit:
    """

    This is a base class to be inherited by a derived Xideco application.

    To import use:

        from xideco.xidekit.xidekit import XideKit

    Methods that may be overwritten:  the __init__ method, the receive_loop and incoming_message_processing
    """

    def __init__(self, router_ip_address=None, subscriber_port='43125', publisher_port='43124'):
        """
        The __init__ method sets up all the ZeroMQ "plumbing"

        :param router_ip_address: Xideco Router IP Address - if not specified, it will be set to the local computer
        :param subscriber_port: Xideco router subscriber port. This must match that of the Xideco router
        :param publisher_port: Xideco router publisher port. This must match that of the Xideco router
        :return:
        """

        # If no router address was specified, determine the IP address of the local machine
        if router_ip_address:
            self.router_ip_address = router_ip_address
        else:
            s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
            # use the google dns
            s.connect(('8.8.8.8', 0))
            self.router_ip_address = s.getsockname()[0]

        print('\n**************************************')
        print('Using router IP address: ' + self.router_ip_address)
        print('**************************************')

        self.subscriber_port = subscriber_port
        self.publisher_port = publisher_port

        # establish the zeriomq sub and pub sockets
        self.context = zmq.Context()
        self.subscriber = self.context.socket(zmq.SUB)
        connect_string = "tcp://" + self.router_ip_address + ':' + self.subscriber_port
        self.subscriber.connect(connect_string)

        self.publisher = self.context.socket(zmq.PUB)
        connect_string = "tcp://" + self.router_ip_address + ':' + self.publisher_port
        self.publisher.connect(connect_string)

    def set_subscriber_topic(self, topic):
        """
        This method sets the subscriber topic.

        You can subscribe to multiple topics by calling this method for
        each topic.
        :param topic: A topic string
        :return:
        """
        if not type(topic) is str:
            raise TypeError('Subscriber topic must be a string')

        self.subscriber.setsockopt(zmq.SUBSCRIBE, topic.encode())

    def publish_payload(self, payload, topic=''):
        """
        This method will publish a payload with the specified topic.

        :param payload: A dictionary of items
        :param topic: A string value
        :return:
        """
        if not type(topic) is str:
            raise TypeError('Publish topic must be a string', 'topic')

        if not type(payload) is dict:
            raise TypeError('Publish payload must be a dictionary', payload)

        # create a message pack payload
        message = umsgpack.packb(payload)

        pub_envelope = topic.encode()
        self.publisher.send_multipart([pub_envelope, message])

    def receive_loop(self):
        """
        This is the receive loop for zmq messages.

        It is assumed that this method will be overwritten to meet the needs of the application and to handle
        received messages.
        :return:
        """
        while True:
            try:
                data = self.subscriber.recv_multipart(zmq.NOBLOCK)
                self.incoming_message_processing(data[0].decode(), umsgpack.unpackb(data[1]))
                time.sleep(.001)
            except zmq.error.Again:
                time.sleep(.001)
            except KeyboardInterrupt:
                self.clean_up()

    # noinspection PyMethodMayBeStatic
    def incoming_message_processing(self, topic, payload):
        """
        Override this method with a message processor for the application

        :param topic: Message Topic string
        :param payload: Message Data
        :return:
        """
        print('this method should be overwritten in the child class', topic, payload)

    def clean_up(self):
        """
        Clean up before exiting - override if additional cleanup is necessary

        :return:
        """
        self.publisher.close()
        self.subscriber.close()
        self.context.term()
        sys.exit(0)



# this is a typical invocation of the class. The class is instantiated, topics are subscribed to, and
# the receive loop is started.
if __name__ == '__main__':
    a = XideKit()
    a.set_subscriber_topic('a')
    a.receive_loop()



#!/usr/bin/env python3
"""
Created on February 15 10:29:15 2016

@author: Alan Yorinks
Copyright (c) 2016 Alan Yorinks All right reserved.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public
License as published by the Free Software Foundation; either
version 3 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""
import math
import sys
import threading
import time

import umsgpack
# noinspection PyPackageRequirements
import zmq
from xideco.data_files.port_map import port_map


# noinspection PyMethodMayBeStatic,PyUnresolvedReferences,PyUnresolvedReferences,PyUnresolvedReferences,PyShadowingNames
class ADXL345(threading.Thread):
    """
        This class allows control and monitoring of the ADXL345 Accelerometer. It is a Xideco Universal i2c
        implementation that will allow use of the ADXL345 on Arduino, Raspberry Pi, and BeagleBone MCUs.
        Range is set to 16 g.

        It uses a separate thread to affect continuous reading.

        At the bottom of this file is a demo program of its use.

        Import this module into your application  and then instantiate to use.

        This module will report:
            board number
            raw x,y,z values
            accelerometer values for x,y,z expressed in g's
            normalized value for x,y,z
            pitch and roll angles
        """

    # This is the pseudo board number indicating that the commands are broadcast to all connectedADXL345
    # simultaneously
    BROADCAST = 5000

    # report data formats
    RAW = 0

    def __init__(self, router_ip_address, board_number=BROADCAST, data_publish_envelope=None):
        """
        :param router_ip_address: IP Address of router
        :param board_number: Board number or BROADCAST to send to all boards
        :param: publish_envelope: Set this parameter to publish data using the user specified envelope
        :return:
        """
        # call the parent class init and set the thread as a daemon thread
        super().__init__()
        self.daemon = True

        # router IP address
        self.board_number = board_number
        self.data_publish_envelope = data_publish_envelope
        self.read_time = None

        # A parameter is provided by the user to force the router ip address instead of reading from the
        # configuration file.

        # if not specified, the router ip address is read from the configuration file
        if router_ip_address == 'None':
            self.router_ip_address = port_map.port_map['router_ip_address']
        else:
            self.router_ip_address = router_ip_address

        print('\n************************************************************')
        print('ADXL345')
        print('Using this router IP address: ' + self.router_ip_address)
        print('************************************************************')

        # establish the zeriomq subscriber socket
        self.context = zmq.Context()
        self.subscriber = self.context.socket(zmq.SUB)
        connect_string = 'tcp://' + self.router_ip_address + ':' + port_map.port_map['subscribe_to_router_port']
        self.subscriber.connect(connect_string)

        # create the topic we wish to subscribe to - we will accept messages from all boards without regard to
        # board number. The reported data will contain the board number being reported
        env_string = 'B'
        envelope = env_string.encode()
        self.subscriber.setsockopt(zmq.SUBSCRIBE, envelope)

        # create the zeromq publisher socket
        self.publisher = self.context.socket(zmq.PUB)
        connect_string = 'tcp://' + self.router_ip_address + ':' + port_map.port_map['publish_to_router_port']
        self.publisher.connect(connect_string)

        # Published message topics - either a Q for broadcast or A + board number for individual boards
        if board_number < self.BROADCAST:
            self.publish_envelope = ("A" + str(board_number)).encode()
        else:
            self.publish_envelope = ("Q".encode())

        # The Xideco protocol message received
        self.payload = None

        # user callback method called to optionally report data
        self.callback = None

        # a flag that the user can set via start_continuous and stop_continuous to affect continuous reading of data
        self.keep_reading = True

        # the last read is stored here and can be retrieved by a call to get_last_data
        self.last_data = {'board': 0, 'x_raw': 0, 'y_raw': 0, 'z_raw': 0,
                          'x_g': 0.0, 'y_g': 0.0, 'z_g': 0.0,
                          'x_a': 0.0, 'y_a': 0.0, 'z_a': 0.0,
                          'pitch': 0.0, 'roll': 0.0}

        # this is the device i2c xideco operations descriptor

        # initialization:
        #   Send and "init" instruction
        #   Set the Power Control Register to 0
        #   Set the Power Control Register to 8 = Measure Bit enabled
        #   Set the Data Format Register to 8 for full resolution
        #   set the Data Format Resister to 3 - 10 bit resolution and 16 g range

        # read
        #   Retrieve 6 bytes from the Data Register

        self.i2c_device = {
            "adxl345": {
                "device_address": 83,
                "commands": {
                    "init": [
                        {
                            u"cmd": u"init",
                        },
                        {
                            u"cmd": u"write_byte",
                            u"register": 45,
                            u"value": 0
                        }, {
                            u"cmd": u"write_byte",
                            u"register": 45,
                            u"value": 8
                        }, {
                            u"cmd": u"write_byte",
                            u"register": 49,
                            u"value": 8
                        },
                        {
                            u"cmd": u"write_byte",
                            u"register": 49,
                            u"value": 3
                        }
                    ],
                    "read": [{
                        u"cmd": "read_block",
                        u"register": 50,
                        u"num_bytes": 6
                    }]
                }
            }
        }

    def initialize_device(self, callback=None):
        """
        This method sends out the i2c initialization sequence to the MCUs. If a callback is specified, all
        data will be reported via the callback, as well as stored internally to allow polling.
        :param callback: Optional callback function provided by user
        :return:
        """
        if callback:
            self.callback = callback
        msg = self.i2c_device['adxl345']['commands']['init']
        self._send_to_device(msg)

    def read_device(self):
        """
        Force a one-shot read of the device
        :return: Return the last set of values read
        """
        msg = self.i2c_device['adxl345']['commands']['read']
        self._send_to_device(msg)
        self._process_subscribed_message()
        return self.last_data

    def start_continuous(self, read_time=None):
        """
        Start a continuous read. Create the read thread if not already created.
        If read time is None, time period between read is .001 seconds
        :param read_time: Amount of time to delay after receiving data back from device
        :return:
        """
        self.keep_reading = True
        self.read_time = read_time

        # start the thread if it is not running already
        if not self.is_alive():
            self.start()

    def stop_continuous(self):
        """
        This function halts the automatic read and reporting of the device
        :return:
        """
        self.keep_reading = False

    def get_last_data(self):
        """
        This method retrieves the data retrieved from the last read - continuous or manual
        :return:
        """
        return self.last_data

    def _send_to_device(self, msg):
        """
        Send a control messages to the device
        :param msg: A list of messages to be sent as specified by i2c_device
        :return:
        """
        address = self.i2c_device['adxl345']['device_address']

        for i in msg:
            i.update({u"command": u"i2c_request"})
            i.update({u"device_address": address})

            msg = umsgpack.packb(i)
            time.sleep(.001)
            self.publisher.send_multipart([self.publish_envelope, msg])

    def _process_subscribed_message(self):
        """
        Wait for data to be returned by the i2c device.
        It stores the data in last_data, and calls the callback if one was specified when the device was initialized.
        :return:
        """
        # noinspection PyPep8Naming
        EARTH_GRAVITY_MS2 = 9.80665

        while True:
            if self.keep_reading:
                # noinspection PyBroadException
                try:
                    msg = self.subscriber.recv_multipart(zmq.NOBLOCK)
                    # extract board number from envelope
                    board = msg[0].decode()
                    board = int(board[1:])
                    self.payload = umsgpack.unpackb(msg[1])

                    if not 'data' in self.payload:
                        continue
                    raw = self.payload['data']

                    # mask off bits not being used
                    raw[1] &= 0x3
                    raw[3] &= 0x3
                    raw[5] &= 0x3

                    # get raw value for x
                    x = raw[0] | (raw[1] << 8)
                    x = self._twos_comp(x, 10)
                    x = round(x, 4)

                    # express x in g forces
                    xg = x * .004
                    xg = round(xg, 4)

                    # express x as a normalized acceleration
                    xa = xg * EARTH_GRAVITY_MS2
                    xa = round(xa, 4)

                    # get raw value for y
                    y = raw[2] | (raw[3] << 8)
                    y = self._twos_comp(y, 10)
                    y = round(y, 4)

                    # express y in g forces
                    yg = y * .004
                    yg = round(yg, 4)

                    # express y as a normalized acceleration
                    ya = yg * EARTH_GRAVITY_MS2
                    ya = round(ya, 4)

                    # get raw value for z
                    z = raw[4] | (raw[5] << 8)
                    z = self._twos_comp(z, 10)
                    z = round(z, 4)

                    # express z in g forces
                    zg = z * .004
                    zg = round(zg, 4)

                    # express z as a normalized acceleration
                    za = zg * EARTH_GRAVITY_MS2
                    za = round(za, 4)

                    pitch = int(-(math.atan2(xa, math.sqrt(ya * ya + za * za)) * 180.0) / math.pi)

                    roll = int((math.atan2(ya, za) * 180.0) / math.pi)

                    self.last_data[u'x_raw'] = x
                    self.last_data[u'y_raw'] = y
                    self.last_data[u'z_raw'] = z

                    self.last_data[u'x_g'] = xg
                    self.last_data[u'y_g'] = yg
                    self.last_data[u'z_g'] = zg

                    self.last_data[u'x_a'] = xa
                    self.last_data[u'y_a'] = ya
                    self.last_data[u'z_a'] = za

                    self.last_data[u'roll'] = roll
                    self.last_data[u'pitch'] = pitch

                    self.last_data[u'board'] = board

                    # if there is a callback registered, call it
                    if self.callback:
                        self.callback(self.last_data)

                    # if there is a publisher envelope, publish the data
                    if self.data_publish_envelope:
                        data_publish_envelope = (self.data_publish_envelope.encode())

                        msg = umsgpack.packb(self.last_data)
                        time.sleep(.001)
                        self.publisher.send_multipart([data_publish_envelope, msg])
                        time.sleep(.001)

                    return
                except KeyboardInterrupt:
                    self.clean_up()
                    sys.exit(0)
                except zmq.error.Again:
                    time.sleep(.001)
            else:
                time.sleep(.001)
                pass

    def _twos_comp(self, val, bits):
        if (val & (1 << (bits - 1))) != 0:
            val -= 1 << bits
        return val

    def clean_up(self):
        """
        Clean things up on exit
        :return:
        """
        self.subscriber.close()
        self.publisher.close()
        self.context.term()

    def run(self):
        while True:
            try:
                msg = self.i2c_device['adxl345']['commands']['read']
                self._send_to_device(msg)
                self._process_subscribed_message()
                if self.read_time:
                    time.sleep(self.read_time)
            except KeyboardInterrupt:
                self.clean_up()
                sys.exit()
            except SystemExit:
                self.clean_up()
                sys.exit()


# a sample callback routine
def data_callback(data):
    p = data['pitch']
    r = data['roll']
    print('Pitch: {0}, Roll: {1}.'.format(p, r))
    # print('Pitch: {0}, Roll: {1}.'.format(r, p))
    # print(data['x_raw'], data['y_raw'], data['z_raw'])
    # print(data['x_a'], data['y_a'], data['z_a'])


def adxl345(router_address):
    # instantiate the device forcing the routing IP address and setting the data publisher envelope to 'z'
    device = ADXL345(router_address, data_publish_envelope='z')

    try:

        # poll data for 10 iterations
        print("polling 10 times")

        # first initialize the device
        device.initialize_device()

        # now poll ten times and print to the console
        for x in range(0, 10):
            device.read_device()
            print(device.get_last_data())

        print("continuous callback for 3 seconds")

        # to run continuous, initialize the device specifying a call back function
        device.initialize_device(data_callback)

        # start continuous operation with a delay after data reported of .001 seconds
        device.start_continuous(.001)

        time.sleep(3)

        # we can halt the continuous operation by calling stop_continuous
        print('halting for 3 seconds')
        device.stop_continuous()
        time.sleep(3)

        # We can re-enable by calling start_continuous
        print('enabling continuous for 3 seconds')
        device.start_continuous(.001)
        time.sleep(3)

        # Halt and then read the last data reported
        print('halting and reading the last value reported then wait 2 seconds')
        device.stop_continuous()
        print(device.get_last_data())
        time.sleep(2)

        # we are out of here
        print('exiting')
        device.clean_up()
        sys.exit(0)
    except KeyboardInterrupt:
        device.clean_up()
        sys.exit(0)


if __name__ == "__main__":
    adxl345("192.163.2.193")



#!/usr/bin/env python3
"""
Created on January 9 11:39:15 2016

@author: Alan Yorinks
Copyright (c) 2016 Alan Yorinks All right reserved.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public
License as published by the Free Software Foundation; either
version 3 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

"""
This file contains a dictionary specifying various IP port numbers used with ZeroMQ sockets.
"""

# This map contains the ip address of the computer currently running the router.
# This address must be manually modified to match the IP address of the router computer.
# All other entries are fixed and should not be modifified.

# The publish_to_router_port should be used by all entities that wish to publish information.
# the subscribe_to_router_port should be used by all entities that with to subscribe to messages. The
# subscribers need to set a topic filter to receive the messages of interest.


port_map = {"router_ip_address": "192.168.2.191",
            "publish_to_router_port": "43124", "subscribe_to_router_port": "43125"}



#!/usr/bin/env python3

# 2014-08-26 PCF8591.py

import time
import curses

import pigpio

# sudo pigpiod
# ./PCF8591.py

# Connect Pi 3V3 - VCC, Ground - Ground, SDA - SDA, SCL - SCL.

YL_40=0x48

pi = pigpio.pi() # Connect to local Pi.

handle = pi.i2c_open(1, YL_40, 0)

# stdscr = curses.initscr()

# curses.noecho()
# curses.cbreak()

aout = 0

# stdscr.addstr(10, 0, "Brightness")
# stdscr.addstr(12, 0, "Temperature")
# stdscr.addstr(14, 0, "AOUT->AIN2")
# stdscr.addstr(16, 0, "Resistor")

# stdscr.nodelay(1)

# try:
while True:

  for a in range(0,4):
     aout = aout + 1
     pi.i2c_write_byte_data(handle, 0x40 | ((a+1) & 0x03), aout&0xFF)
     v = pi.i2c_read_byte(handle)
     # hashes = v / 4
     # spaces = 64 - hashes
     # stdscr.addstr(10+a*2, 12, str(v) + ' ')
     # stdscr.addstr(10+a*2, 16, '#' * hashes + ' ' * spaces )
     print('a: ' + str(a) + 'v:' + str(v))

  # stdscr.refresh()
  time.sleep(0.04)

      # c = stdscr.getch()

      # if c != curses.ERR:
      #    break

# except:


# curses.nocbreak()
#  curses.echo()
# curses.endwin()

pi.i2c_close(handle)
pi.stop()



#!/usr/bin/env python

# morse_code.py
# 2015-06-17
# Public Domain

# ./morse_code.py [ message [ micros [ gpio ] ] ]

import sys
import time

import pigpio

morse={
'a':'.-'   , 'b':'-...' , 'c':'-.-.' , 'd':'-..'  , 'e':'.'    ,
'f':'..-.' , 'g':'--.'  , 'h':'....' , 'i':'..'   , 'j':'.---' ,
'k':'-.-'  , 'l':'.-..' , 'm':'--'   , 'n':'-.'   , 'o':'---'  ,
'p':'.--.' , 'q':'--.-' , 'r':'.-.'  , 's':'...'  , 't':'-'    ,
'u':'..-'  , 'v':'...-' , 'w':'.--'  , 'x':'-..-' , 'y':'-.--' ,
'z':'--..' , '1':'.----', '2':'..---', '3':'...--', '4':'....-',
'5':'.....', '6':'-....', '7':'--...', '8':'---..', '9':'----.',
'0':'-----'}

NONE=0

DASH=3
DOT=1

GAP=1
LETTER_GAP=3-GAP
WORD_GAP=7-LETTER_GAP

text = """Now is the winter of our discontent
made glorious summer by this sun of York"""

if len(sys.argv) > 1:
   text = sys.argv[1]

micros = 20000

if len(sys.argv) > 2:
   micros = int(sys.argv[2])

GPIO=18

if len(sys.argv) > 3:
   GPIO = int(sys.argv[3])

def transmit_string(pi, gpio, message):

   print(message)

   pi.wave_clear() # clear all waveforms

   wf=[]

   for C in message:
      c=C.lower()
      if c in morse:
         k = morse[c]
         for x in k:

            if x == '.':
               wf.append(pigpio.pulse(1<<gpio, NONE, DOT * micros))
            else:
               wf.append(pigpio.pulse(1<<gpio, NONE, DASH * micros))

            wf.append(pigpio.pulse(NONE, 1<<gpio, GAP * micros))

         wf.append(pigpio.pulse(NONE, 1<<gpio, LETTER_GAP * micros))

      elif c == ' ':
         wf.append(pigpio.pulse(NONE, 1<<gpio, WORD_GAP * micros))

   pi.wave_add_generic(wf)

   wid = pi.wave_create()

   if wid >= 0:
      pi.wave_send_once(wid)

pi = pigpio.pi() # connect to local Pi

pi.set_mode(GPIO, pigpio.OUTPUT)

transmit_string(pi, GPIO, text)

while pi.wave_tx_busy():
   time.sleep(0.1)

pi.stop() # disconnect from Pi



#!/usr/bin/env python3
"""
Created on January 9 11:39:15 2016

@author: Alan Yorinks
Copyright (c) 2016 Alan Yorinks All right reserved.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public
License as published by the Free Software Foundation; either
version 3 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

import signal

import umsgpack
# noinspection PyPackageRequirements
import zmq
import time
import subprocess
import sys

from xideco.data_files.port_map import port_map


# noinspection PyMethodMayBeStatic,PyUnresolvedReferences,PyUnresolvedReferences,PyUnresolvedReferences
class TwitterBridge:
    """
    The Twitter Bridge provides the protocol bridge between Xideco and a Twitter generation request from Scratch
    """

    def __init__(self):
        """
        :param pymata_board: Pymata-aio instance
        :param board_num: Arduino Board Number (1-10)
        :return:
        """

        # establish the zeriomq sub and pub sockets
        self.context = zmq.Context()
        self.subscriber = self.context.socket(zmq.SUB)
        connect_string = "tcp://" + port_map.port_map['router_ip_address'] + ':' + port_map.port_map[
            'subscribe_to_router_port']
        self.subscriber.connect(connect_string)

        # create the topic we wish to subscribe to
        env_string = "A100" + self.board_num
        envelope = env_string.encode()
        self.subscriber.setsockopt(zmq.SUBSCRIBE, envelope)

        self.publisher = self.context.socket(zmq.PUB)
        connect_string = "tcp://" + port_map.port_map['router_ip_address'] + ':' + port_map.port_map[
            'publish_to_router_port']

        self.publisher.connect(connect_string)

        self.payload = ""

    def run_twitter_bridge(self):

        # noinspection PyBroadException
        try:
            msg = self.subscriber.recv_multipart(zmq.NOBLOCK)
            self.payload = umsgpack.unpackb(msg[1])
            print("[%s] %s" % (msg[0], self.payload))
            message = self.payload["message"]
            print(message)

            command = 'twitter -eMisterYsLab@gmail.com set %s' % message
            subprocess.call(command, shell=True)
            time.sleep(.001)
        except zmq.error.Again:
            time.sleep(.001)

    def clean_up(self):
        """
        Clean things up on exit
        :return:
        """
        self.subscriber.close()
        self.publisher.close()
        self.context.term()


def twitter_bridge():
    # noinspection PyShadowingNames

    # create an instance of the twitter bridge and then run it
    tbridge = TwitterBridge()
    while True:
        tbridge.run_twitter_bridge()

    # signal handler function called when Control-C occurs
    # noinspection PyShadowingNames,PyUnusedLocal,PyUnusedLocal
    def signal_handler(signal, frame):
        print("Control-C detected. See you soon.")

        tbridge.clean_up()

        sys.exit(0)

    # listen for SIGINT
    signal.signal(signal.SIGINT, signal_handler)
    signal.signal(signal.SIGTERM, signal_handler)


if __name__ == "__main__":

    try:
        twitter_bridge()
    except KeyboardInterrupt:
        sys.exit(0)



#!/usr/bin/env python2
"""
Created on February 16 08:48:15 2016

@author: Alan Yorinks
Copyright (c) 2016 Alan Yorinks All right reserved.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public
License as published by the Free Software Foundation; either
version 3 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

import argparse
import time
# noinspection PyPackageRequirements
import signal
import sys
import zmq
import umsgpack

from Adafruit_I2C import Adafruit_I2C


# noinspection PyMethodMayBeStatic,PyUnresolvedReferences,PyUnresolvedReferences,PyUnresolvedReferences
class BeagleBoneI2CBridge:
    """
    The Raspberry Bridge provides the protocol bridge between Xideco and a Raspberry Pi board using the
    pigpio library https://github.com/joan2937/pigpio

    """

    def __init__(self, board_num, router_ip_address, publisher_socket, subscriber_socket):
        """
        :param board_num: System Board Number (1-10)
        :param board_type: "black" or "green"
        :param servo_polarity: 1 or 0
        :return:
        """
        self.board_num = board_num
        if router_ip_address == 'None':
            print('You must use the -r command line option to specify the router IP Address')
            sys.exit(0)
        else:
            self.router_ip_address = router_ip_address

        self.publisher_socket = publisher_socket
        self.subscriber_socket = subscriber_socket

        print('\n*****************************************')
        print('BeagleBone Black i2c Interface - xibbi2c')
        print('Using router IP address: ' + self.router_ip_address)
        print('Publisher Socket: ' + self.publisher_socket)
        print('Subscriber Socket: ' + self.subscriber_socket)
        print('*****************************************')

        # establish the zeriomq sub and pub sockets
        self.context = zmq.Context()
        self.subscriber = self.context.socket(zmq.SUB)
        connect_string = "tcp://" + self.router_ip_address + ':' + self.subscriber_socket
        self.subscriber.connect(connect_string)

        # create the topic we wish to subscribe to
        env_string = "A" + self.board_num
        envelope = env_string.encode()
        self.subscriber.setsockopt(zmq.SUBSCRIBE, envelope)
        # subscribe to broadcast i2c message - i2c messages can also be board specific with A + board number
        self.subscriber.setsockopt(zmq.SUBSCRIBE, 'Q'.encode())

        self.publisher = self.context.socket(zmq.PUB)
        connect_string = "tcp://" + self.router_ip_address + ':' + self.publisher_socket

        self.publisher.connect(connect_string)

        # The Xideco protocol message received
        self.payload = None

        self.i2c_handle_dict = {}

    def i2c_request(self):
        cmd = self.payload['cmd']
        addr = self.payload['device_address']

        if cmd == 'init':
            handle = Adafruit_I2C(addr)
            self.i2c_handle_dict.update({addr: handle})
        elif cmd == 'write_byte':
            # get handle
            value = self.payload['value']
            handle = self.i2c_handle_dict[addr]
            register = self.payload['register']
            handle.write8(register, value)
        elif cmd == 'read_block':
            num_bytes = self.payload['num_bytes']
            handle = self.i2c_handle_dict[addr]
            register = self.payload['register']
            data = handle.readList(register, num_bytes)
            time.sleep(.0001)
            self.report_i2c_data(data)
        else:
            print('unknown cmd')

    def report_i2c_data(self, data):
        # create a topic specific to the board number of this board
        envelope = ("B" + self.board_num).encode()

        msg = umsgpack.packb({u"command": "i2c_reply", u"board": self.board_num, u"data": data})

        self.publisher.send_multipart([envelope, msg])

    def run_bb_i2c_bridge(self):
        """
        Start up the bridge
        :return:
        """
        # self.pi.set_mode(11, pigpio.INPUT)
        # cb1 = self.pi.callback(11, pigpio.EITHER_EDGE, self.cbf)
        while True:
            # noinspection PyBroadException
            try:
                z = self.subscriber.recv_multipart(zmq.NOBLOCK)
                self.payload = umsgpack.unpackb(z[1])
                # print("[%s] %s" % (z[0], self.payload))

                # print(self.payload)
                command = self.payload['command']
                if command == 'i2c_request':
                    self.i2c_request()
                else:
                    # print("can't execute unknown command'")
                    pass
                time.sleep(.0001)
            except KeyboardInterrupt:
                sys.exit(0)
            except zmq.error.Again:
                time.sleep(.0001)


def beaglebone_i2c_bridge():
    # noinspection PyShadowingNames

    parser = argparse.ArgumentParser()
    parser.add_argument("-b", dest="board_number", default="1", help="Board Number - 1 through 10")
    parser.add_argument("-p", dest="publisher_socket", default="43124", help="Publisher Socket Number")
    parser.add_argument("-s", dest="subscriber_socket", default="43125", help="Publisher Socket Number")
    parser.add_argument("-r", dest="router_ip_address", default="None", help="Router IP Address")

    args = parser.parse_args()

    board_num = args.board_number
    router_ip_address = args.router_ip_address
    publisher_socket = args.publisher_socket
    subscriber_socket = args.subscriber_socket

    bb_bridge = BeagleBoneI2CBridge(board_num, router_ip_address, publisher_socket,
                                    subscriber_socket)
    try:
        bb_bridge.run_bb_i2c_bridge()
    except KeyboardInterrupt:
        sys.exit(0)

    # signal handler function called when Control-C occurs
    # noinspection PyShadowingNames,PyUnusedLocal,PyUnusedLocal
    def signal_handler(signal, frame):
        print("\nControl-C detected. See you soon.")
        sys.exit(0)

    # listen for SIGINT
    signal.signal(signal.SIGINT, signal_handler)
    signal.signal(signal.SIGTERM, signal_handler)


if __name__ == "__main__":
    beaglebone_i2c_bridge()



"""

   Pubsub envelope subscriber

   Author: Guillaume Aubert (gaubert) <guillaume(dot)aubert(at)gmail(dot)com>

"""
import zmq

def main():
    """ main method """

    # Prepare our context and publisher
    context    = zmq.Context()
    subscriber = context.socket(zmq.SUB)
    subscriber.connect("tcp://localhost:5563")
    subscriber.setsockopt(zmq.SUBSCRIBE, b"A")

    while True:
        # Read envelope with address
        [address, contents] = subscriber.recv_multipart()
        print("[%s] %s" % (address, str(contents)))

    # We never get here but clean up anyhow
    subscriber.close()
    context.term()


if __name__ == "__main__":
    main()



"""

   Pubsub envelope subscriber

   Author: Guillaume Aubert (gaubert) <guillaume(dot)aubert(at)gmail(dot)com>

"""
import zmq
import sys

def main():
    """ main method """

    # Prepare our context and publisher
    context = zmq.Context()
    subscriber = context.socket(zmq.SUB)
    subscriber.connect("tcp://localhost:5563")
    subscriber.setsockopt(zmq.SUBSCRIBE, b"B")

    while True:
        # Read envelope with address
        try:
            [address, contents] = subscriber.recv_multipart(zmq.NOBLOCK)
        except zmq.error.Again:
            #raise
            sys.exit(0)

        if address:
            print("[%s] %s" % (address, contents))

    # We never get here but clean up anyhow
    subscriber.close()
    context.term()


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
"""
Created on February 15 10:29:15 2016

@author: Alan Yorinks
Copyright (c) 2016 Alan Yorinks All right reserved.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public
License as published by the Free Software Foundation; either
version 3 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

# This is a sample program to monitor data being sent from adxl345.py using 'z' as the envelope.

# The router IP address is hardcoded as well as the port number
import umsgpack
import zmq
import time
import sys


context = zmq.Context()
subscriber = context.socket(zmq.SUB)
connect_string = 'tcp://192.168.2.192:43125'
subscriber.connect(connect_string)

env_string = 'z'
envelope = env_string.encode()
subscriber.setsockopt(zmq.SUBSCRIBE, envelope)

while True:
    try:
        msg = subscriber.recv_multipart(zmq.NOBLOCK)
        payload = umsgpack.unpackb(msg[1])
        print(payload)
    except KeyboardInterrupt:
        subscriber.close()
        context.term()
        sys.exit(0)
    except zmq.error.Again:
        time.sleep(.001)




import pigpio
import time

servos = 25  # GPIO number
# delay = .7

pi = pigpio.pi()
pi.set_mode(25, pigpio.OUTPUT)

try:

    while True:
       pi.write(25, 1)
       time.sleep(0.0015)
       pi.write(25, 0)
       time.sleep(0.0185)
except KeyboardInterrupt:
    pi.set_servo_pulsewidth(servos, 0)

pi.stop()



import time

import pigpio

servos = 25  # GPIO number
delay = .6

pi = pigpio.pi()
# pulsewidth can only set between 500-2500
print(pi.get_PWM_frequency(25))
print(pi.get_PWM_range(25))

pi.set_PWM_frequency(25, 800)
pi.set_PWM_range(25, 2500)

print(pi.get_PWM_frequency(25))
print(pi.get_PWM_range(25))
try:
    while True:
        # pi.set_servo_pulsewidth(servos, 1000)  # 0 degree
        # print("Servo {} {} micro pulses".format(str(servos), 500))
        # time.sleep(delay)
        # pi.set_servo_pulsewidth(servos, 0)
        # time.sleep(1)



        pi.set_servo_pulsewidth(servos, 500)  # 0 degree
        # print("Servo {} {} micro pulses".format(str(servos), 500))
        time.sleep(delay)
        pi.set_servo_pulsewidth(servos, 0)
        # time.sleep(.5)

        pi.set_servo_pulsewidth(servos, 2500)  # 0 degree
        # print("Servo {} {} micro pulses".format(str(servos), 1000))
        time.sleep(delay)
        pi.set_servo_pulsewidth(servos, 0)
        # time.sleep(.5)


        # switch all servos off
except KeyboardInterrupt:
    pi.set_servo_pulsewidth(servos, 0)

pi.stop()



from pymata_aio.pymata3 import PyMata3
from pymata_aio.constants import Constants
import zmq
import umsgpack




class Abridge():
    def __init__(self, board):
        self.board = board

    def cb(self,data):
        print(data)

    def do_command(self, msg):
        x = umsgpack.unpackb(msg)
        board.set_pin_mode(int(x['pin']), x['mode'], self.cb)



command = "digital_pin_mode"
# board = request.match_info.get('board')
enable = 'enable'
pin = '12'
mode = Constants.INPUT
command_msg = umsgpack.packb({u"command": command, u"enable": enable, u"pin": pin, u"mode": mode})

board = PyMata3()
bridge = Abridge(board)
bridge.do_command(command_msg)
while True:
    board.sleep(1)


import zmq
import random
import sys
import time

port = "5666"
context = zmq.Context()
socket = context.socket(zmq.PAIR)
# socket.connect("tcp://localhost:%s" % port)
socket.connect("tcp://*:%s" % port)

#socket.bind("tcp://*:%s" % port)

while True:
    # msg = socket.recv_string()
    # print (msg)
    socket.send_multipart([b"B", b"We would like to see this"])
    # socket.send_string("client message to server1")
    # socket.send_string("client message to server2")
    time.sleep(1)



import pigpio
import time

pi = pigpio.pi()

pi.set_mode(4, pigpio.OUTPUT)

print(pi.get_PWM_range(4))
print(pi.get_PWM_real_range(4))


# pi.write(4,1)
# time.sleep(2)
# pi.write(4,0)
# time.sleep(1)

pi.set_PWM_dutycycle(4, 20) # PWM 3/4 on

time.sleep(3)
pi.stop()


from setuptools import setup

setup(
        name='xideco',
        version='0.5.0',
        packages=['xideco', 'xideco.data_files', 'xideco.data_files.port_map', 'xideco.data_files.configuration',
                  'xideco.data_files.scratch_files', 'xideco.data_files.scratch_files.projects',
                  'xideco.data_files.scratch_files.extensions', 'xideco.http_bridge', 'xideco.xideco_router',
                  'xideco.arduino_bridge', 'xideco.raspberrypi_bridge','xideco.beaglebone_bridge',
                  'experiments', 'experiments.xideco_tweeter','xideco.i2c.i2c_devices.adxl345',
                  'xideco.xidekit'],
        install_requires=['pymata-aio>=2.8',
                          'aiohttp>=0.19.0',
                          'pyzmq>=15.1.0',
                          'umsgpack>=0.1.0',
                          'Adafruit_BBIO'],
        package_data={'xideco.data_files': [('configuration/*'),
                                            ('scratch_files/extensions/*.s2e'),
                                            ('scratch_files/projects/*.sb2')]},
        entry_points={
            'console_scripts': [
                'xiab = xideco.arduino_bridge.xiab:arduino_bridge',
                'xihb = xideco.http_bridge.xihb:http_bridge',
                'xirt = xideco.xideco_router.xirt:xideco_router',
                'xirb = xideco.raspberrypi_bridge.xirb:raspberrypi_bridge',
                'xibb = xideco.beaglebone_bridge.xibb:beaglebone_bridge',
                'xibbi2c = xideco.beaglebone_bridge.xibbi2c:beaglebone_bridge'
            ]
        },
        url='https://github.com/MrYsLab/xideco/wiki',
        license='GNU General Public License v3 (GPLv3)',
        author='Alan Yorinks',
        author_email='MisterYsLab@gmail.com',
        description='A Network Enabled Software Backplane',
        keywords=['Firmata', 'Arduino', 'Scratch', 'ZeroMQ'],
        classifiers=[
            'Development Status :: 4 - Beta',
            'Environment :: Other Environment',
            'Intended Audience :: Education',
            'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
            'Operating System :: OS Independent',
            'Programming Language :: Python :: 3.5',
            'Topic :: Education',
            'Topic :: Software Development',
            'Topic :: Home Automation',
            'Topic :: System :: Hardware'
        ],
)



import time  # import the time library

import pigpio  # import the GPIO library

pin = 25

# establish 2 pulse definitions to create a 1k hz tone
# the first entry sets pin 18 high for a duration of 1000 microseconds
# and the second entry set pin 18 low for a duration of 1000 microseconds
tone_1k = [pigpio.pulse(1 << pin, 0, 1000), pigpio.pulse(0, 1 << pin, 1000)]


# remember to execute "sudo pigpiod" on the Pi before executing

#instantiate pigpio
pi = pigpio.pi()
pi.set_mode(pin, pigpio.INPUT)
time.sleep(1)
pi.set_mode(pin, pigpio.OUTPUT)
delay_period = 1

# set the buzzer pin to output
#pi.set_mode(pin, pigpio.OUTPUT)
#pi.set_mode(17, pigpio.INPUT)

while True:

    # for angle in range(600, 2400):
    #     pi.set_servo_pulsewidth(pin, angle)
    #     time.sleep(delay_period)
    # for angle in range(2400, 600):
    #     pi.set_servo_pulsewidth(pin, angle)
    #     time.sleep(delay_period)

    pi.set_servo_pulsewidth(pin, 500)
    time.sleep(.2)
    pi.set_servo_pulsewidth(pin, 0)

    time.sleep(1)

    pi.set_servo_pulsewidth(pin, 1500)
    time.sleep(.2)
    pi.set_servo_pulsewidth(pin, 0)

    time.sleep(1)



import time  # import the time library

import pigpio  # import the GPIO library

pin = 18

# establish 2 pulse definitions to create a 1k hz tone
# the first entry sets pin 18 high for a duration of 1000 microseconds
# and the second entry set pin 18 low for a duration of 1000 microseconds
tone_1k = [pigpio.pulse(1 << pin, 0, 1000), pigpio.pulse(0, 1 << pin, 1000)]


# remember to execute "sudo pigpiod" on the Pi before executing

#instantiate pigpio
pi = pigpio.pi()

# set the buzzer pin to output
pi.set_mode(pin, pigpio.OUTPUT)

# clear any previous wave descriptions
pi.wave_clear()

# create a wave descriptor
pi.wave_add_generic(tone_1k)  # 100 ms flashes

# create the tone and save the id
tone = pi.wave_create()

# play tone continuously for one second
pi.wave_send_repeat(tone)
time.sleep(1)
pi.wave_tx_stop()  # stop waveform

#clean up
pi.wave_clear()  # clear all waveforms
pi.write(pin, 0)



import zmq
import time

context = zmq.Context()
subscriber = context.socket(zmq.SUB)

subscriber.connect('tcp://192.168.2.199:43125')

try:
    z = subscriber.recv_multipart(zmq.NOBLOCK)
except zmq.error.Again:
    print('ok')
    time.sleep(1)

print('done')



import time  # import the time library

import pigpio  # import the GPIO library


class Buzzer(object):
    def __init__(self):
        # GPIO.setmode(GPIO.BCM)
        self.pi = pigpio.pi()
        self.buzzer_pin = 18  # set to GPIO pin 5
        # GPIO.setup(self.buzzer_pin, GPIO.IN)
        # GPIO.setup(self.buzzer_pin, GPIO.OUT)
        self.pi.set_mode(self.buzzer_pin, pigpio.INPUT)

        self.pi.set_mode(self.buzzer_pin, pigpio.OUTPUT)
        print("buzzer ready")

    def __del__(self):
        class_name = self.__class__.__name__
        print(class_name, "finished")

    def buzz(self, pitch, duration):  # create the function “buzz” and feed it the pitch and duration)

        if pitch == 0:
            time.sleep(duration)
            return
        period = 1.0 / pitch  # in physics, the period (sec/cyc) is the inverse of the frequency (cyc/sec)
        delay = period / 2  # calculate the time for half of the wave
        cycles = int(duration * pitch)  # the number of waves to produce is the duration times the frequency

        for i in range(cycles):  # start a loop from 0 to the variable “cycles” calculated above
            self.pi.write(self.buzzer_pin, 1)  # set pin 18 to high
            time.sleep(delay)  # wait with pin 18 high
            self.pi.write(self.buzzer_pin, 0)  # set pin 18 to low
            time.sleep(delay)  # wait with pin 18 low

    def play(self, tune):
        # GPIO.setmode(GPIO.BCM)
        self.pi.set_mode(self.buzzer_pin, pigpio.OUTPUT)
        x = 0

        print("Playing tune ", tune)
        if tune == 1:
            pitches = [262, 294, 330, 349, 392, 440, 494, 523, 587, 659, 698, 784, 880, 988, 1047]
            duration = 0.1
            for p in pitches:
                self.buzz(p, duration)  # feed the pitch and duration to the function, “buzz”
                time.sleep(duration * 0.5)
            for p in reversed(pitches):
                self.buzz(p, duration)
                time.sleep(duration * 0.5)

        elif tune == 2:
            pitches = [262, 330, 392, 523, 1047]
            duration = [0.2, 0.2, 0.2, 0.2, 0.2, 0, 5]
            for p in pitches:
                self.buzz(p, duration[x])  # feed the pitch and duration to the function, “buzz”
                time.sleep(duration[x] * 0.5)
                x += 1
        elif tune == 3:
            pitches = [392, 294, 0, 392, 294, 0, 392, 0, 392, 392, 392, 0, 1047, 262]
            duration = [0.2, 0.2, 0.2, 0.2, 0.2, 0.2, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.8, 0.4]
            for p in pitches:
                self.buzz(p, duration[x])  # feed the pitch and duration to the func$
                time.sleep(duration[x] * 0.5)
                x += 1

        elif tune == 4:
            pitches = [1047, 988, 659]
            duration = [0.1, 0.1, 0.2]
            for p in pitches:
                self.buzz(p, duration[x])  # feed the pitch and duration to the func$
                time.sleep(duration[x] * 0.5)
                x += 1

        elif tune == 5:
            pitches = [1047, 988, 523]
            duration = [0.1, 0.1, 0.2]
            for p in pitches:
                self.buzz(p, duration[x])  # feed the pitch and duration to the func$
                time.sleep(duration[x] * 0.5)
                x += 1

        self.pi.set_mode(self.buzzer_pin, pigpio.INPUT)


if __name__ == "__main__":
    a = input("Enter Tune number 1-5:")
    buzzer = Buzzer()
    buzzer.play(int(a))



#!/usr/bin/env python3
"""
Created on January 9 11:39:15 2016

@author: Alan Yorinks
Copyright (c) 2016 Alan Yorinks All right reserved.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public
License as published by the Free Software Foundation; either
version 3 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

import argparse
import time

import pigpio
import umsgpack

# noinspection PyPackageRequirements
import zmq
from xideco.data_files.port_map import port_map

import signal
import sys
import threading


# noinspection PyMethodMayBeStatic,PyUnresolvedReferences,PyUnresolvedReferences,PyUnresolvedReferences
class RaspberryPiBridge:
    """
    The Raspberry Bridge provides the protocol bridge between Xideco and a Raspberry Pi board using the
    pigpio library https://github.com/joan2937/pigpio

    """

    def __init__(self, pi, board_num, router_ip_address):
        """
        :param pigpio: pigpio instance
        :param board_num: System Board Number (1-10)
        :return:
        """
        self.pi = pi

        # there are 3 types of raspberry pi boards dependent upon rev number:
        # http://elinux.org/RPi_HardwareHistory#Board_Revision_History

        # create a list of rev number to board type
        self.type_1_revs = [2, 3]
        self.type_2_revs = [4, 5, 6, 7, 8, 9, 13, 14, 15]
        # type_3 is >= 16 - this value will be used to compare against
        self.type_3_revs = 16

        # each board type supports a different set of GPIOs
        self.type_1_invalid_pins = [2, 3, 5, 6, 12, 13, 16, 19, 20, 26, 27, 28, 29, 30, 31]
        self.type_2_invalid_pins = [0, 1, 5, 6, 12, 13, 16, 19, 20, 21, 26]
        self.type_3_invalid_pins = [0, 1, 28, 29, 30, 31]

        # put them all in one list for simpler access
        self.unavailable_pins = [self.type_1_invalid_pins, self.type_2_invalid_pins,
                                 self.type_3_invalid_pins]

        # board type - 1, 2 or 3
        self.pi_board_type = None

        self.router_ip_address = router_ip_address

        # a list of dictionary items, each entry is a single pin
        # dictionary entries are {mode, enabled}
        self.pins = []

        # set a list called pins to hold the pin modes
        for x in range(0, 32):
            mode = pi.get_mode(x)
            entry = {'mode': mode, 'enabled': False}
            self.pins.append(entry)

        # save user supplied board number
        self.board_num = board_num

        print("board num: " + board_num)

        # determine the hardware rev of the board
        hw_rev = self.pi.get_hardware_revision()
        print('HW REV: ' + str(hw_rev))

        # use that to set the board type
        if hw_rev >= self.type_3_revs:
            self.pi_board_type = 3
        elif hw_rev in self.type_1_revs:
            self.pi_board_type = 1
        elif hw_rev in self.type_2_revs:
            self.pi_board_type = 2
        else:
            print("Unknown Hardware Rev: " + hw_rev)
            sys.exit(0)

        print('Raspberry Pi Board Type Detected: ' + str(self.pi_board_type))

        pigpio_ver = self.pi.get_pigpio_version()
        print('PIGPIO REV: ' + str(pigpio_ver))

        if self.router_ip_address == 'None':
            self.router_ip_address = port_map.port_map['router_ip_address']
        else:
            self.router_ip_address = router_ip_address

        print('Xideco Raspberry Pi Bridge - xirp')
        print('\n**************************************')
        print('Using router IP address: ' + self.router_ip_address)
        print('**************************************')

        print('\nTo specify some other address for the router, use the -r command line option')

        # establish the zeriomq sub and pub sockets
        self.context = zmq.Context()
        self.subscriber = self.context.socket(zmq.SUB)
        connect_string = "tcp://" + self.router_ip_address + ':' + port_map.port_map['subscribe_to_router_port']
        self.subscriber.connect(connect_string)

        # create the topic we wish to subscribe to
        env_string = "A" + self.board_num
        envelope = env_string.encode()
        self.subscriber.setsockopt(zmq.SUBSCRIBE, envelope)
        # subscribe to broadcast i2c message - i2c messages can also be board specific with A + board number
        self.subscriber.setsockopt(zmq.SUBSCRIBE, 'Q'.encode())

        self.publisher = self.context.socket(zmq.PUB)
        connect_string = "tcp://" + self.router_ip_address + ':' + port_map.port_map['publish_to_router_port']

        self.publisher.connect(connect_string)

        # The Xideco protocol message received
        self.payload = None

        self.command_dict = {'digital_pin_mode': self.setup_digital_pin, 'digital_write': self.digital_write,
                             'analog_pin_mode': self.setup_analog_pin, 'analog_write': self.analog_write,
                             'set_servo_position': self.set_servo_position, 'play_tone': self.play_tone,
                             'tone_off': self.tone_off, 'i2c_request': self.i2c_request}

        self.last_problem = ''

        self.sonar = None
        self.a_to_d = None

        self.i2c_handle_dict = {}

    def setup_analog_pin(self):
        """
        This method validates and configures a pin for analog input
        :return: None
        """
        # clear out any residual problem strings
        #

        self.last_problem = '2-0\n'

        if not self.a_to_d:
            self.a_to_d = AtoD(self.pi, 1, 0x48, self.board_num)
            self.a_to_d.start()

        # test pin range 0-3

        pin = int(self.payload['pin'])
        if 0 <= pin <= 3:
            # check if enable or disable
            enable = self.payload['enable']
            if enable == 'Enable':
                self.a_to_d.set_report(pin, True)
            else:
                self.a_to_d.set_report(pin, False)

        else:
            self.last_problem = '2-1\n'

    def setup_digital_pin(self):
        """
        This method processes the Scratch "Digital Pin" Block that establishes the mode for the pin
        :return: None
        """

        # clear out any residual problem strings
        # self.report_problem('1-0\n')

        self.last_problem = '1-0\n'

        # retrieve mode from command
        mode = self.payload['mode']

        # Is the user enabling or disabling the pin? Get the 'raw' value and translate it.
        enable = self.payload['enable']

        if mode == 'SONAR':
            pins = self.payload['pin'].split('.')
            self.payload['pin'] = pins[0]
            trigger_pin = self.validate_pin()
            if trigger_pin == 99:
                self.last_problem = '1-2\n'
                return

            self.payload['pin'] = pins[1]
            echo_pin = self.validate_pin()
            if echo_pin == 99:
                self.last_problem = '1-3\n'
                return

            if enable == 'Enable':
                self.enable_sonar(trigger_pin, echo_pin)
            else:
                self.disable_sonar()

        else:

            pin = self.validate_pin()
            if pin == 99:
                self.last_problem = '1-1\n'
                return

            # retrieve mode from command
            mode = self.payload['mode']
            #
            if enable == 'Enable':
                # validate the mode for this pin
                if mode == 'Input':
                    self.pi.set_mode(pin, pigpio.INPUT)

                    # update the pin table
                    pin_entry = {'mode': pigpio.INPUT, 'enabled': True}
                    self.pins[pin] = pin_entry
                    self.pi.callback(pin, pigpio.EITHER_EDGE, self.cbf)
                elif mode == 'Output':
                    self.pi.set_mode(pin, pigpio.OUTPUT)

                    # update the pin table
                    pin_entry = {'mode': pigpio.OUTPUT, 'enabled': True}
                    self.pins[pin] = pin_entry

                elif mode == 'PWM':
                    self.pi.set_mode(pin, pigpio.OUTPUT)

                    # update the pin table
                    pin_entry = {'mode': pigpio.OUTPUT, 'enabled': True}
                    self.pins[pin] = pin_entry

                elif mode == 'Tone':
                    self.pi.set_mode(pin, pigpio.OUTPUT)

                    # update the pin table
                    pin_entry = {'mode': pigpio.OUTPUT, 'enabled': True}
                    self.pins[pin] = pin_entry

                elif mode == 'Servo':
                    self.pi.set_mode(pin, pigpio.OUTPUT)

                    # update the pin table
                    pin_entry = {'mode': pigpio.OUTPUT, 'enabled': True}
                    self.pins[pin] = pin_entry
                    self.pi.set_mode(pin, pigpio.OUTPUT)

            # must be disable
            else:
                pin_entry = self.pins[pin]
                pin_entry['enabled'] = False
                self.pins[pin] = self.pin_entry

    def analog_write(self):
        """
        Set a PWM configured pin to the requested value
        :return: None
        """

        # clear out any residual problem strings
        self.last_problem = '4-0\n'

        pin = self.validate_pin()
        if pin == 99:
            self.last_problem = '4-1\n'
            return

        # get pin information
        pin_state = self.pins[pin]
        if pin_state['mode'] != pigpio.OUTPUT:
            self.last_problem = '4-2\n'
            return

        if not pin_state['enabled']:
            self.last_problem = '4-3\n'
            return

        value = int(self.payload['value'])

        if not (0 <= value <= 255):
            selt.last_problem = '4-4\n'
            return

        # self.board.digital_write(pin, value)
        self.pi.set_PWM_dutycycle(pin, value)

    def digital_write(self):
        """
        Set the state of a digital pin
        :return:
        """
        # clear out any residual problem strings
        self.last_problem = '3-0\n'

        pin = self.validate_pin()
        if pin == 99:
            self.last_problem = '3-1\n'
            return

        # get pin information
        pin_state = self.pins[pin]
        if pin_state['mode'] != pigpio.OUTPUT:
            self.last_problem = '3-2\n'
            return

        # if not pin_state['enabled']:
        #     self.last_problem = '3-3\n'
        #     return

        value = int(self.payload['value'])
        #     self.board.digital_write(pin, value)
        self.pi.write(pin, value)

    def play_tone(self):
        """
        This method will play a tone using a wave.
        :return: None
        """
        # clear out any residual problem strings
        self.last_problem = '5-0\n'

        pin = self.validate_pin()
        if pin == 99:
            self.last_problem = '5-1\n'
            return

        # get pin information
        pin_state = self.pins[pin]
        if pin_state['mode'] != pigpio.OUTPUT:
            self.last_problem = '5-2\n'
            return
        frequency = int((1000 / int(self.payload['frequency'])) * 1000)
        duration = int(self.payload['duration'])

        tone = [pigpio.pulse(1 << pin, 0, frequency), pigpio.pulse(0, 1 << pin, frequency)]  # flash every 100 ms

        self.pi.wave_clear()

        self.pi.wave_add_generic(tone)  # 100 ms flashes
        tone_wave = self.pi.wave_create()  # create and save id
        self.pi.wave_send_repeat(tone_wave)

        if duration == 0:
            return

        sleep_time = duration * .001
        time.sleep(sleep_time)
        self.pi.wave_tx_stop()  # stop waveform

        self.pi.wave_clear()  # clear all waveforms

    def tone_off(self):
        self.last_problem = '6-0\n'

        pin = self.validate_pin()
        if pin == 99:
            self.last_problem = '6-1\n'
            return

        self.pi.wave_tx_stop()  # stop waveform

        self.pi.wave_clear()  # clear all waveforms

    def set_servo_position(self):
        """
        Set a servo position
       :return:
        """

        # time to allow servo to move
        delay = .6

        self.last_problem = '7-0\n'

        pin = self.validate_pin()
        if pin == 99:
            self.last_problem = '7-1\n'
            return

        # get pin information
        pin_state = self.pins[pin]
        if pin_state['mode'] != pigpio.OUTPUT:
            self.last_problem = '7-2\n'
            return

        position = int(self.payload['position'])

        # range of values for position is 500 to 2500
        # each degree is approximately equal to 11   (2000/180)
        # we add this to 500, the zero degree position
        position = (position * 11) + 500

        self.pi.set_servo_pulsewidth(pin, position)  # 0 degree
        time.sleep(delay)
        self.pi.set_servo_pulsewidth(pin, 0)

    def i2c_request(self):
        cmd = self.payload['cmd']
        addr = self.payload['device_address']

        if cmd == 'init':
            handle = self.pi.i2c_open(1, addr)
            self.i2c_handle_dict.update({addr: handle})
        elif cmd == 'write_byte':
            # get handle
            value = self.payload['value']
            handle = self.i2c_handle_dict[addr]
            register = self.payload['register']
            self.pi.i2c_write_byte_data(handle, register, value)
        elif cmd == 'read_block':
            num_bytes = self.payload['num_bytes']
            handle = self.i2c_handle_dict[addr]
            register = self.payload['register']
            data = self.pi.i2c_read_i2c_block_data(handle, register, num_bytes)
            self.report_i2c_data(data)
        else:
            print('unknown cmd')

    def report_i2c_data(self, data):
        # create a topic specific to the board number of this board
        envelope = ("B" + self.board_num).encode()

        # extract bytes from returned byte array and place in a list
        rdata = []
        for x in data[1]:
            rdata.append(x)

        msg = umsgpack.packb({u"command": "i2c_reply", u"board": self.board_num, u"data": rdata})

        self.publisher.send_multipart([envelope, msg])

    def run_raspberry_bridge(self):
        # self.pi.set_mode(11, pigpio.INPUT)
        # cb1 = self.pi.callback(11, pigpio.EITHER_EDGE, self.cbf)
        while True:
            if self.last_problem:
                self.report_problem()
            # noinspection PyBroadException
            try:
                z = self.subscriber.recv_multipart(zmq.NOBLOCK)

                self.payload = umsgpack.unpackb(z[1])

                command = self.payload['command']
                if command in self.command_dict:
                    self.command_dict[command]()
                else:
                    print("can't execute unknown command'")
                    # time.sleep(.001)
            except KeyboardInterrupt:
                self.cleanup()
                sys.exit(0)
            except zmq.error.Again:
                time.sleep(.001)

    def enable_sonar(self, trigger, echo):
        self.sonar = Sonar(self.pi, trigger, echo, self.board_num)
        self.sonar.start()

    def disable_sonar(self):
        if self.sonar:
            self.sonar.cancel()

    def report_problem(self):
        """
        Publish the supplied Xideco protocol message
        :return: None
        """
        # create a topic specific to the board number of this board
        envelope = ("B" + self.board_num).encode()

        msg = umsgpack.packb({u"command": "problem", u"board": 1, u"problem": self.last_problem})

        self.publisher.send_multipart([envelope, msg])
        self.last_problem = ''

    # noinspection PyUnusedLocal
    def cbf(self, gpio, level, tick):
        """
        Gpio callback method to report changes
        :param gpio: pin
        :param level: value
        :param tick: time stamp
        :return:
        """

        # if the pin has reports disabled, just ignore
        pin_state = self.pins[gpio]

        # if user changes modes suppress output from being sent upstream
        if pin_state['mode'] == pigpio.OUTPUT:
            return
        digital_reply_msg = umsgpack.packb({u"command": "digital_read", u"pin": str(gpio), u"value": str(level)})

        envelope = ("B" + self.board_num).encode()
        self.publisher.send_multipart([envelope, digital_reply_msg])

    def cleanup(self):
        print('\nExiting...')
        self.pi.stop()

    def validate_pin(self):
        """
        This method validates a pin number
        :return: Pin number if valid, 99 if invalid
        """
        # payload is established in run_raspberry_bridge
        try:
            pin = int(self.payload['pin'])
        except ValueError:
            # Pin Must Be Specified as an Integer 1-1
            return 99

        # check that pin does not exceed maximum BCM GPIO number
        if pin > 31:
            return 99

        # validate the gpio number for the board in use
        if pin in self.unavailable_pins[self.pi_board_type - 1]:
            return 99

        return pin


class Sonar(threading.Thread):
    """
    This class encapsulates a type of acoustic ranger.  In particular
    the type of ranger with separate trigger and echo pins.

    A pulse on the trigger initiates the sonar ping and shortly
    afterwards a sonar pulse is transmitted and the echo pin
    goes high.  The echo pins stays high until a sonar echo is
    received (or the response times-out).  The time between
    the high and low edges indicates the sonar round trip time.
    """

    def __init__(self, rpi, trigger, echo, board_num):
        """
        The class is instantiated with the Pi to use and the
        gpios connected to the trigger and echo pins.
        """
        super().__init__()
        self.pi = rpi
        self._trig = trigger
        # self._trig = 22
        self._echo = echo
        # self._echo = 9

        self.board_num = board_num

        self._ping = False
        self._high = None
        self._time = None

        self._triggered = False

        self._trig_mode = self.pi.get_mode(self._trig)
        self._echo_mode = self.pi.get_mode(self._echo)

        self.pi.set_mode(self._trig, pigpio.OUTPUT)
        self.pi.set_mode(self._echo, pigpio.INPUT)

        self._cb = self.pi.callback(self._trig, pigpio.EITHER_EDGE, self._cbf)
        self._cb = self.pi.callback(self._echo, pigpio.EITHER_EDGE, self._cbf)

        self.context = zmq.Context()

        self.publisher = self.context.socket(zmq.PUB)
        connect_string = "tcp://" + port_map.port_map['router_ip_address'] + ':' + port_map.port_map[
            'reporter_publisher_port']

        self.publisher.connect(connect_string)

        self._inited = True

    def _cbf(self, gpio, level, tick):
        """
        Callback from sonar data changes
        :param gpio: pin
        :param level: value
        :param tick: timestamp
        :return:
        """
        if gpio == self._trig:
            if level == 0:  # trigger sent
                self._triggered = True
                self._high = None
        else:
            if self._triggered:
                if level == 1:
                    self._high = tick
                else:
                    if self._high is not None:
                        self._time = tick - self._high
                        self._high = None
                        self._ping = True

    def read(self):
        """
        Triggers a reading.  The returned reading is the number
        of microseconds for the sonar round-trip.

        round trip cms = round trip time / 1000000.0 * 34030
        """
        if self._inited:
            try:
                self._ping = False
                self.pi.gpio_trigger(self._trig)
                start = time.time()
                while not self._ping:
                    if (time.time() - start) > 5.0:
                        return 20000
                    time.sleep(0.001)
                return self._time
            except AttributeError:
                sys.exit(0)
        else:
            return None

    def cancel(self):
        """
        Cancels the ranger and returns the gpios to their
        original mode.
        """
        if self._inited:
            self._inited = False
            self._cb.cancel()
            self.pi.set_mode(self._trig, self._trig_mode)
            self.pi.set_mode(self._echo, self._echo_mode)

    def run(self):
        """
        Retrieve sonar data and send report
        :return:
        """

        if not self._inited:
            self.cancel()

        end = time.time() + 600.0

        while time.time() < end:
            x = self.read()
            if x:
                # calculate round trip time
                x = x / 1000000.0 * 34030.0

                # calculate distance and round it off
                x /= 2
                x = round(x, 2)

                # publish the data

                digital_reply_msg = umsgpack.packb({u"command": "digital_read", u"pin": str(self._trig),
                                                    u"value": str(x)})

                envelope = ("B" + self.board_num).encode()
                self.publisher.send_multipart([envelope, digital_reply_msg])
                time.sleep(0.03)


class AtoD(threading.Thread):
    """
    This class handles the pcf8591 YL 40 Module analog to digital conversion module
    """

    def __init__(self, rpi, bus, address, board_num):
        """

        :param rpi: pigpio instance
        :param bus: i2c bus
        :param address: i2c address
        :return: nothing is returned
        """
        super().__init__()
        self.pi = rpi
        self.bus = bus
        self.address = address
        self.board_num = board_num

        self.handle = self.pi.i2c_open(self.bus, self.address)
        self.context = zmq.Context()

        self.publisher = self.context.socket(zmq.PUB)
        connect_string = "tcp://" + port_map.port_map['router_ip_address'] + ':' + port_map.port_map[
            'reporter_publisher_port']

        self.publisher.connect(connect_string)

        self.reports = [False, False, False, False]

    def set_report(self, index, value):
        """
        Set report table to send or deny reports
        :param index: pin number index into table
        :param value: True or False
        :return: nothing returned
        """
        self.reports[index] = value

    def run(self):
        """
        Continuously monitor the A/D
        :return:
        """
        a_out = 0
        while True:
            for a in range(0, 4):
                if self.reports[a]:
                    a_out += 1
                    self.pi.i2c_write_byte_data(self.handle, 0x40 | ((a + 1) & 0x03), a_out & 0xFF)
                    v = self.pi.i2c_read_byte(self.handle)
                    digital_reply_msg = umsgpack.packb({u"command": "analog_read", u"pin": str(a),
                                                        u"value": str(v)})
                    envelope = ("B" + self.board_num).encode()
                    self.publisher.send_multipart([envelope, digital_reply_msg])
            time.sleep(0.04)


def raspberrypi_bridge():
    """
    Main function for the raspberry pi bridge
    :return:
    """
    # noinspection PyShadowingNames

    parser = argparse.ArgumentParser()
    parser.add_argument("-b", dest="board_number", default="1", help="Board Number - 1 through 10")
    parser.add_argument('-r', dest='router_ip_address', default='None', help='Router IP Address')

    args = parser.parse_args()

    pi = pigpio.pi()

    board_num = args.board_number
    router_ip_address = args.router_ip_address
    rpi_bridge = RaspberryPiBridge(pi, board_num, router_ip_address)
    try:
        rpi_bridge.run_raspberry_bridge()
    except KeyboardInterrupt:
        rpi_bridge.cleanup()
        pi.stop()
        sys.exit(0)

    # signal handler function called when Control-C occurs
    # noinspection PyShadowingNames,PyUnusedLocal,PyUnusedLocal
    def signal_handler(signal, frame):
        print("\nControl-C detected. See you soon.")
        sys.exit(0)

    # listen for SIGINT
    signal.signal(signal.SIGINT, signal_handler)
    signal.signal(signal.SIGTERM, signal_handler)


if __name__ == "__main__":
    raspberrypi_bridge()
